/**********************************************************************
 * 
 *  File: 
 * 
 *  Created:
 *
 *  Version: $Id: FrameBuffer.hpp,v 1.18 2005/11/25 11:18:15 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef _SHALLOWS_FRAMEBUFFER_HPP
#define _SHALLOWS_FRAMEBUFFER_HPP
#include "shallows/config.h"

namespace shallows {

class Texture2D;	
class RenderTarget;

/** \brief Abstract base class for handling framebuffers.
 *  A framebuffer consist of a collection of render targets.
 *  
 *  All functions (including constructor) may set this framebuffer
 *  as current framebuffer.
 *
*/
class  DllExport FrameBuffer
{
public:
   FrameBuffer();
   virtual ~FrameBuffer();
   //   virtual void attachRenderTarget(RenderTarget &r, int colBufId=-1) = 0;

   /** Validates whether a render target is attachable to the framebuffer.
    *  Otherwise std::invalid_argument is thrown. 
    * \param r The render target to validate.
    */
   virtual void validateRenderTarget(const shallows::RenderTarget &r) const =0;

   /** Activates rendering to the framebuffer. This function does \b not
    *  change the viewport. If multiple framebuffers with different sizes
    *  are used the application is responsible for setting the viewport
    *  according to the size of the framebuffer.
    */
   virtual void activate() = 0;

   /** Checks that the framebuffer is ready to be rendered into.
    * Otherwise frame_buffer_incomplete is thrown */
   virtual void validateCompleteness() const = 0;

   //   virtual const unsigned int *drawBuffers(int n) const = 0;
   
   //   virtual unsigned int getDrawBuffer(int n) const = 0;

protected:
  private:
  
   void operator=(const FrameBuffer &b);
};
} // namespace shallows

#endif // _SHALLOWS_FRAMEBUFFER_HPP

