/**********************************************************************
 * 
 *  File: OffScreenRenderTarget.hpp
 * 
 *  Created: 18. July 2005
 *
 *  Version: $Id: OffScreenRenderTarget.hpp,v 1.8 2005/11/25 11:18:15 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	         Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	         Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef _OFFSCREENRENDERTARGET_HPP_
#define _OFFSCREENRENDERTARGET_HPP_
#include "shallows/config.h"

#include "shallows/RenderTarget.hpp"
#include <boost/shared_ptr.hpp>

namespace shallows {
   class OffScreenBuffer;
   
  /** \brief Abstract class for handling off screen RenderTargets.
   */
class  DllExport OffScreenRenderTarget : public RenderTarget
{
   friend unsigned int attach(OffScreenBuffer *off_buff, 
		      boost::shared_ptr<shallows::OffScreenRenderTarget> &rt, 
		      int colAttachable);

   friend void attach(OffScreenBuffer *off_buff, 
		     boost::shared_ptr<shallows::OffScreenRenderTarget> *rt, 
		     unsigned int *enums,
		     unsigned int numBuffs);
public:

  OffScreenRenderTarget();
  virtual ~OffScreenRenderTarget();

   /** Binds the rendertarget to the current framebuffer.
    *  \param colAttachment The color attachment point to bind the
    *  framebuffer to. If none is given it is bound to the same
    *  affachment point as previous.
    */
   virtual void bindToCurrentFrameBuffer(int colAttachment=-1) = 0; 

   /** Binds the rendertarget as a depth buffer to the current framebuffer.
    */
   virtual void bindAsDepthToCurrentFrameBuffer() = 0;

   /** Returns the with of the render target. */
  virtual unsigned int getWidth() const = 0;

   /** Returns the height of the render target. */
  virtual unsigned int getHeight() const = 0;

   /** Returns the internalformat of the render target. */
  virtual int getInternalFormat() const = 0;

protected:
   virtual void attachToCurrentFrameBuffer(int colAttachment) = 0; 

   mutable int prevColAttachment_;

private:
}; // 

} // end namespace shallows


#endif // _OFFSCREENRENDERTARGET_HPP_
