/**********************************************************************
 * 
 *  File: 
 * 
 *  Created:
 *
 *  Version: $Id: OnScreenBuffer.hpp,v 1.12 2005/11/25 11:18:15 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef _SHALLOWS_ONSCREENBUFFER_HPP
#define _SHALLOWS_ONSCREENBUFFER_HPP
#include "shallows/config.h"
#include <vector>
#include "shallows/FrameBuffer.hpp"
namespace shallows {

class Texture2D;	
class RenderTarget;

/** \brief Handles rendering to window-system-provided framebuffer
 *  (i.e. the onscreen framebuffer). This class enables the use of
 *  shallows::Program for rendering to the screen, and not only to 
 *  offscreen framebuffers. 
 */
#pragma warning( disable: 4251 )

class  DllExport OnScreenBuffer : public FrameBuffer
{
public:
   OnScreenBuffer();
   virtual ~OnScreenBuffer();
   virtual void validateRenderTarget(const shallows::RenderTarget &r) const;
   virtual void activate();

   /** Disables framebuffer objects */
   static void setActive();
   virtual void validateCompleteness() const;

  private:
  
   void operator=(const OnScreenBuffer &b);
   std::vector<unsigned int> drawBuffers_;
};
} // namespace shallows

#endif // _SHALLOWS_ONSCREENBUFFER_HPP

