/**********************************************************************
 * 
 *  File: RenderBuffer.hpp
 * 
 *  Created:
 *
 *  Version: $Id: RenderBuffer.hpp,v 1.8 2005/11/25 11:18:15 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef _RENDERBUFFER_HPP_
#define _RENDERBUFFER_HPP_
#include "shallows/config.h"

#include <boost/shared_ptr.hpp>
#include "shallows/OffScreenRenderTarget.hpp"

namespace shallows {

   /** \brief Offscreen renter target which is never used
    *  as texture.
    *
    *  Dependent of the internal format it can be used either 
    *  as a color buffer or as a depth buffer.  
    */
class  DllExport RenderBuffer : public OffScreenRenderTarget
{

public:
   /** Creates a renderbuffer.
    *  \param internalFormat The internal format for the Renderbuffer.
    *  \param width The width of the buffer.
    *  \param height The height of the buffer.
    */
   RenderBuffer(int internalFormat,
		unsigned int width,
		unsigned int height);
   virtual ~RenderBuffer();

   virtual void bindToCurrentFrameBuffer(int colAttachment); 
   virtual void bindAsDepthToCurrentFrameBuffer();
   virtual unsigned int getLastBufferEnum() const;

  unsigned int getWidth() const;
  unsigned int getHeight() const;
  int getInternalFormat() const;

protected:
   virtual void attachToCurrentFrameBuffer(int colAttachment); 
private:
  RenderBuffer() {};
   void operator=(const RenderBuffer &b) {};
   unsigned int bufId_;
   int internalFormat_;
   int width_, height_;
}; // 

} // end namespace shallows


#endif // _RENDERBUFFER_HPP_
