/**********************************************************************
 * 
 *  File: RenderTexture2D.hpp
 * 
 *  Created:
 *
 *  Version: $Id: RenderTexture2D.hpp,v 1.16 2005/11/25 11:18:15 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef _RENDERTEXTURE2D_HPP_
#define _RENDERTEXTURE2D_HPP_
#include "shallows/config.h"

#include <boost/shared_ptr.hpp>
#include "shallows/Texture2D.hpp"
#include "shallows/OffScreenRenderTarget.hpp"

#pragma warning( disable: 4251 )

namespace shallows {

   /** \brief Offscreen render target which can be used
    *  as a texture.
    *
    *  Dependent of the internal format it can be used either 
    *  as a color buffer or as a depth buffer.  
    */
class  DllExport RenderTexture2D : public OffScreenRenderTarget
{

public:
   typedef boost::shared_ptr<shallows::Texture2D> texture_ptr;
   /** Creates a RenderTexture which renders into an existing
    *  texture. */
   RenderTexture2D(texture_ptr tex);
   virtual ~RenderTexture2D();

   virtual void bindToCurrentFrameBuffer(int colAttachment); 
   virtual void bindAsDepthToCurrentFrameBuffer();
   virtual unsigned int getLastBufferEnum() const;

  unsigned int getWidth() const;
  unsigned int getHeight() const;
  int getInternalFormat() const;
   
   /** Returns a pointer to the texture which can be used to read the
    *  render target.*/
  texture_ptr getTexture() const;

protected:
   virtual void attachToCurrentFrameBuffer(int colAttachment); 
private:
  RenderTexture2D() {};
   void operator=(const RenderTexture2D &b) {};
   boost::shared_ptr<shallows::Texture2D> texture_;
}; // 

} // end namespace shallows


#endif // _RENDERTEXTURE2D_HPP_
