/**********************************************************************
 * 
 *  File: Texture.hpp
 * 
 *  Created: 04. July 2005
 *
 *  Version: $Id: Texture.hpp,v 1.7 2005/09/26 15:07:00 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	         Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	         Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef _TEXTURE_HPP_
#define _TEXTURE_HPP_
#include "shallows/config.h"

#include <string>
namespace shallows {

   /** \brief Abstract base class for handling textures.
    */
class  DllExport Texture 
{

public:
   virtual ~Texture();

   /** Returns the OpenGL texture id. */
   unsigned int getTextureID() const;

   /** Binds the texture to the active texture unit. */
   virtual void bind() const = 0;

   /** Return the internal format of the texture. */
   virtual int getInternalFormat() const = 0;

   /** Throws shallows_invalid_argument unless the texture
    *  can be bound to a sampler of type s. */
   virtual void validateSamplerType(const std::string &s) const = 0;
protected:
   /** Creates a texture object */
   Texture(unsigned int textureID=0);
  unsigned int textureID_;
   bool owned_;
private:


}; // Texture2D

} // end namespace Shallows


#endif // _TEXTURE_2D_HPP_
