/**********************************************************************
 * 
 *  File: Texture2D.hpp
 * 
 *  Created: 23. June 2005
 *
 *  Version: $Id: Texture2D.hpp,v 1.10 2005/09/26 12:24:35 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	         Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	         Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef _SHALLOWS_TEXTURE_2D_HPP_
#define _SHALLOWS_TEXTURE_2D_HPP_
#include "shallows/config.h"

#include "shallows/Texture.hpp"
namespace shallows {

   /** \brief Texture2D contains a two dimensional texture.
    *
    *  The texture can either be created by Texture2D
    *  or externally.
    */
class  DllExport Texture2D :public shallows::Texture
{

public:
   /** Creates a texture.
    *  \param internalFormat The GLenum to use as internal format.
    *  \param width The width of the texture. 
    *  \param height The height of the texture. 
    *  \param format The format of the pixel data provided in the 
    *   pixels parameter (typically GL_RGBA).
    *  \param type The data type of the pixel data provided in the 
    *   pixels parameter (typically GL_FLOAT).
    *  \param pixels Pointer to the pixel data (Null pointer specifies
    *   texture with unknown data.
    *  \param rectanguar Specifies whether the texture should be of type 
    *   Texture2D (texture coordiates in the range [0..1]x[0..1]) 
    *   of of type texture rectangle (texture coordiates in the range 
    *   [0..width]x[0..height]).
    */
  Texture2D(int internalFormat, 
	    unsigned int width, 
	    unsigned int height, 
	    int format, 
	    int type, 
	    const void *pixels,
	    bool rectanguar=false);

   /** Creates a Texture2D object using an existing texture. This 
    *  assumes that the texture is not deleted before the Texture2D 
    *  object is destroyed.
    *  \param internalFormat The GLenum to use as internal format.
    *  \param width The width of the texture. 
    *  \param height The height of the texture. 
    *  \param textureID The id of the existing texture.
    *  \param rectanguar Specifies whether the texture should be of type 
    *   Texture2D (texture coordiates in the range [0..1]x[0..1]) 
    *   of of type texture rectangle (texture coordiates in the range 
    *   [0..width]x[0..height]).
    */
  Texture2D(int internalFormat, 
	    unsigned int width, 
	    unsigned int height,
	    unsigned int textureID,
	    bool rectanguar=false);
   virtual ~Texture2D();


   /** Returns true if the texture is a rectangular texture
    *  (texture coordiates in the range 
    *   [0..width]x[0..height]).
    */
   bool isRectangular() const;

   /** Returns the width of the texture */
  unsigned int getWidth() const;
   /** Returns the height of the texture */
  unsigned int getHeight() const;
   virtual void bind() const;
  int getInternalFormat() const;
   void validateSamplerType(const std::string &s) const;

private:

  unsigned int height_;
  unsigned int width_;
   int internalFormat_;
   bool rectanguar_;

}; // Texture2D

} // end namespace Shallows


#endif // _SHALLOWS_TEXTURE_2D_HPP_
