/**********************************************************************
 * 
 *  File: Texture3D.hpp
 * 
 *  Created: 23. June 2005
 *
 *  Version: $Id$
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	         Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	         Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef TEXTURE3D_HPP_
#define TEXTURE3D_HPP_

#include "shallows/config.h"

#include "shallows/Texture.hpp"
namespace shallows {

   /** \brief Texture2D contains a two dimensional texture.
    *
    *  The texture can either be created by Texture2D
    *  or externally.
    */
class  DllExport Texture3D :public shallows::Texture
{

public:
   /** Creates a texture3D object from pixeldata.
    *  \param internalFormat The GLenum to use as internal format.
    *  \param width The width of the texture. 
    *  \param height The height of the texture. 
    *  \param depth The depth of the texture.
    *  \param format The format of the pixel data provided in the 
    *   pixels parameter (typically GL_RGBA).
    *  \param type The data type of the pixel data provided in the 
    *   pixels parameter (typically GL_FLOAT).
    *  \param pixels Pointer to the pixel data (Null pointer specifies
    *   texture with unknown data.
    */
  Texture3D(int internalFormat, 
	    unsigned int width, 
	    unsigned int height, 
	    unsigned int depth, 
	    int format, 
	    int type, 
	    const void *pixels );

   /** Creates a Texture2D object using an existing texture. This 
    *  assumes that the texture is not deleted before the Texture2D 
    *  object is destroyed.
    *  \param internalFormat The GLenum to use as internal format.
    *  \param width The width of the texture. 
    *  \param height The height of the texture. 
    *  \param depth The depth of the texture.
    *  \param textureID The id of the existing texture.
    */
  Texture3D(int internalFormat, 
	    unsigned int width, 
	    unsigned int height,
	    unsigned int depth,
	    unsigned int textureID);
   virtual ~Texture3D();

   /** Returns the width of the texture */
  unsigned int getWidth() const;
   /** Returns the height of the texture */
  unsigned int getHeight() const;
  
  /** Returns the depth of the texture. */
  unsigned int getDepth() const;
  
  virtual void bind() const;
  int getInternalFormat() const;
  void validateSamplerType(const std::string &s) const;

private:

  unsigned int height_;
  unsigned int width_;
  unsigned int depth_;
  int internalFormat_;

}; // Texture3D

} // end namespace Shallows




#endif /*TEXTURE3D_HPP_*/
