/**********************************************************************
 * 
 *  File: 
 * 
 *  Created:
 *
 *  Version: $Id: ShallowsExt.hpp,v 1.8 2005/12/06 14:00:35 trondrh Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	         Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	         Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/


#ifndef _SHALLOWS_EXT_HPP_
#define _SHALLOWS_EXT_HPP_
#include "shallows/config.h"


#ifdef UNIX
#include <GL/gl.h>
#define GL_GLEXT_PROTOTYPES 1
#include "shallows/internal/glext.h"
namespace shallows 
{
   namespace internal // ensure that namespace is defined
  {
  bool initShaderExtensions();
  }
}

#endif

#ifdef _WIN32
#include <windows.h>
#include <GL/gl.h>
#include "shallows/internal/glext.h"

namespace shallows
{

namespace internal
{
bool initShaderExtensions();
bool isShaderExtensionSupported(const char *extension);

bool initShaderExtension(const char *extension);


#ifdef INITGLEXT
#define GLEXTERN
#else
#define GLEXTERN extern
#endif

#ifdef GL_VERSION_2_0
  GLEXTERN PFNGLBLENDEQUATIONSEPARATEPROC glBlendEquationSeparate;
  GLEXTERN PFNGLDRAWBUFFERSPROC glDrawBuffers;
  GLEXTERN PFNGLSTENCILOPSEPARATEPROC glStencilOpSeparate;
  GLEXTERN PFNGLSTENCILFUNCSEPARATEPROC glStencilFuncSeparate;
  GLEXTERN PFNGLSTENCILMASKSEPARATEPROC glStencilMaskSeparate;
  GLEXTERN PFNGLATTACHSHADERPROC glAttachShader;
  GLEXTERN PFNGLBINDATTRIBLOCATIONPROC glBindAttribLocation;
  GLEXTERN PFNGLCOMPILESHADERPROC glCompileShader;
  GLEXTERN PFNGLCREATEPROGRAMPROC glCreateProgram;
  GLEXTERN PFNGLCREATESHADERPROC glCreateShader;
  GLEXTERN PFNGLDELETEPROGRAMPROC glDeleteProgram;
  GLEXTERN PFNGLDELETESHADERPROC glDeleteShader;
  GLEXTERN PFNGLDETACHSHADERPROC glDetachShader;
  GLEXTERN PFNGLDISABLEVERTEXATTRIBARRAYPROC glDisableVertexAttribArray;
  GLEXTERN PFNGLENABLEVERTEXATTRIBARRAYPROC glEnableVertexAttribArray;
  GLEXTERN PFNGLGETACTIVEATTRIBPROC glGetActiveAttrib;
  GLEXTERN PFNGLGETACTIVEUNIFORMPROC glGetActiveUniform;
  GLEXTERN PFNGLGETATTACHEDSHADERSPROC glGetAttachedShaders;
  GLEXTERN PFNGLGETATTRIBLOCATIONPROC glGetAttribLocation;
  GLEXTERN PFNGLGETPROGRAMIVPROC glGetProgramiv;
  GLEXTERN PFNGLGETPROGRAMINFOLOGPROC glGetProgramInfoLog;
  GLEXTERN PFNGLGETSHADERIVPROC glGetShaderiv;
  GLEXTERN PFNGLGETSHADERINFOLOGPROC glGetShaderInfoLog;
  GLEXTERN PFNGLGETSHADERSOURCEPROC glGetShaderSource;
  GLEXTERN PFNGLGETUNIFORMLOCATIONPROC glGetUniformLocation;
  GLEXTERN PFNGLGETUNIFORMFVPROC glGetUniformfv;
  GLEXTERN PFNGLGETUNIFORMIVPROC glGetUniformiv;
  GLEXTERN PFNGLGETVERTEXATTRIBDVPROC glGetVertexAttribdv;
  GLEXTERN PFNGLGETVERTEXATTRIBFVPROC glGetVertexAttribfv;
  GLEXTERN PFNGLGETVERTEXATTRIBIVPROC glGetVertexAttribiv;
  GLEXTERN PFNGLGETVERTEXATTRIBPOINTERVPROC glGetVertexAttribPointerv;
  GLEXTERN PFNGLISPROGRAMPROC glIsProgram;
  GLEXTERN PFNGLISSHADERPROC glIsShader;
  GLEXTERN PFNGLLINKPROGRAMPROC glLinkProgram;
  GLEXTERN PFNGLSHADERSOURCEPROC glShaderSource;
  GLEXTERN PFNGLUSEPROGRAMPROC glUseProgram;
  GLEXTERN PFNGLUNIFORM1FPROC glUniform1f;
  GLEXTERN PFNGLUNIFORM2FPROC glUniform2f;
  GLEXTERN PFNGLUNIFORM3FPROC glUniform3f;
  GLEXTERN PFNGLUNIFORM4FPROC glUniform4f;
  GLEXTERN PFNGLUNIFORM1IPROC glUniform1i;
  GLEXTERN PFNGLUNIFORM2IPROC glUniform2i;
  GLEXTERN PFNGLUNIFORM3IPROC glUniform3i;
  GLEXTERN PFNGLUNIFORM4IPROC glUniform4i;
  GLEXTERN PFNGLUNIFORM1FVPROC glUniform1fv;
  GLEXTERN PFNGLUNIFORM2FVPROC glUniform2fv;
  GLEXTERN PFNGLUNIFORM3FVPROC glUniform3fv;
  GLEXTERN PFNGLUNIFORM4FVPROC glUniform4fv;
  GLEXTERN PFNGLUNIFORM1IVPROC glUniform1iv;
  GLEXTERN PFNGLUNIFORM2IVPROC glUniform2iv;
  GLEXTERN PFNGLUNIFORM3IVPROC glUniform3iv;
  GLEXTERN PFNGLUNIFORM4IVPROC glUniform4iv;
  GLEXTERN PFNGLUNIFORMMATRIX2FVPROC glUniformMatrix2fv;
  GLEXTERN PFNGLUNIFORMMATRIX3FVPROC glUniformMatrix3fv;
  GLEXTERN PFNGLUNIFORMMATRIX4FVPROC glUniformMatrix4fv;
  GLEXTERN PFNGLVALIDATEPROGRAMPROC glValidateProgram;
  GLEXTERN PFNGLVERTEXATTRIB1DPROC glVertexAttrib1d;
  GLEXTERN PFNGLVERTEXATTRIB1DVPROC glVertexAttrib1dv;
  GLEXTERN PFNGLVERTEXATTRIB1FPROC glVertexAttrib1f;
  GLEXTERN PFNGLVERTEXATTRIB1FVPROC glVertexAttrib1fv;
  GLEXTERN PFNGLVERTEXATTRIB1SPROC glVertexAttrib1s;
  GLEXTERN PFNGLVERTEXATTRIB1SVPROC glVertexAttrib1sv;
  GLEXTERN PFNGLVERTEXATTRIB2DPROC glVertexAttrib2d;
  GLEXTERN PFNGLVERTEXATTRIB2DVPROC glVertexAttrib2dv;
  GLEXTERN PFNGLVERTEXATTRIB2FPROC glVertexAttrib2f;
  GLEXTERN PFNGLVERTEXATTRIB2FVPROC glVertexAttrib2fv;
  GLEXTERN PFNGLVERTEXATTRIB2SPROC glVertexAttrib2s;
  GLEXTERN PFNGLVERTEXATTRIB2SVPROC glVertexAttrib2sv;
  GLEXTERN PFNGLVERTEXATTRIB3DPROC glVertexAttrib3d;
  GLEXTERN PFNGLVERTEXATTRIB3DVPROC glVertexAttrib3dv;
  GLEXTERN PFNGLVERTEXATTRIB3FPROC glVertexAttrib3f;
  GLEXTERN PFNGLVERTEXATTRIB3FVPROC glVertexAttrib3fv;
  GLEXTERN PFNGLVERTEXATTRIB3SPROC glVertexAttrib3s;
  GLEXTERN PFNGLVERTEXATTRIB3SVPROC glVertexAttrib3sv;
  GLEXTERN PFNGLVERTEXATTRIB4NBVPROC glVertexAttrib4Nbv;
  GLEXTERN PFNGLVERTEXATTRIB4NIVPROC glVertexAttrib4Niv;
  GLEXTERN PFNGLVERTEXATTRIB4NSVPROC glVertexAttrib4Nsv;
  GLEXTERN PFNGLVERTEXATTRIB4NUBPROC glVertexAttrib4Nub;
  GLEXTERN PFNGLVERTEXATTRIB4NUBVPROC glVertexAttrib4Nubv;
  GLEXTERN PFNGLVERTEXATTRIB4NUIVPROC glVertexAttrib4Nuiv;
  GLEXTERN PFNGLVERTEXATTRIB4NUSVPROC glVertexAttrib4Nusv;
  GLEXTERN PFNGLVERTEXATTRIB4BVPROC glVertexAttrib4bv;
  GLEXTERN PFNGLVERTEXATTRIB4DPROC glVertexAttrib4d;
  GLEXTERN PFNGLVERTEXATTRIB4DVPROC glVertexAttrib4dv;
  GLEXTERN PFNGLVERTEXATTRIB4FPROC glVertexAttrib4f;
  GLEXTERN PFNGLVERTEXATTRIB4FVPROC glVertexAttrib4fv;
  GLEXTERN PFNGLVERTEXATTRIB4IVPROC glVertexAttrib4iv;
  GLEXTERN PFNGLVERTEXATTRIB4SPROC glVertexAttrib4s;
  GLEXTERN PFNGLVERTEXATTRIB4SVPROC glVertexAttrib4sv;
  GLEXTERN PFNGLVERTEXATTRIB4UBVPROC glVertexAttrib4ubv;
  GLEXTERN PFNGLVERTEXATTRIB4UIVPROC glVertexAttrib4uiv;
  GLEXTERN PFNGLVERTEXATTRIB4USVPROC glVertexAttrib4usv;
  GLEXTERN PFNGLVERTEXATTRIBPOINTERPROC glVertexAttribPointer;
#endif

#ifdef GL_VERSION_1_2
  GLEXTERN PFNGLTEXIMAGE3DPROC glTexImage3D;
  GLEXTERN PFNGLTEXSUBIMAGE3DPROC glTexSubImage3D;
  GLEXTERN PFNGLCOPYTEXSUBIMAGE3DPROC glCopyTexSubImage3D;
#endif

#ifdef GL_ARB_shader_objects
  GLEXTERN PFNGLDELETEOBJECTARBPROC glDeleteObjectARB;
  GLEXTERN PFNGLGETHANDLEARBPROC glGetHandleARB;
  GLEXTERN PFNGLDETACHOBJECTARBPROC glDetachObjectARB;
  GLEXTERN PFNGLCREATESHADEROBJECTARBPROC glCreateShaderObjectARB;
  GLEXTERN PFNGLSHADERSOURCEARBPROC glShaderSourceARB;
  GLEXTERN PFNGLCOMPILESHADERARBPROC glCompileShaderARB;
  GLEXTERN PFNGLCREATEPROGRAMOBJECTARBPROC glCreateProgramObjectARB;
  GLEXTERN PFNGLATTACHOBJECTARBPROC glAttachObjectARB;
  GLEXTERN PFNGLLINKPROGRAMARBPROC glLinkProgramARB;
  GLEXTERN PFNGLUSEPROGRAMOBJECTARBPROC glUseProgramObjectARB;
  GLEXTERN PFNGLVALIDATEPROGRAMARBPROC glValidateProgramARB;
  GLEXTERN PFNGLUNIFORM1FARBPROC glUniform1fARB;
  GLEXTERN PFNGLUNIFORM2FARBPROC glUniform2fARB;
  GLEXTERN PFNGLUNIFORM3FARBPROC glUniform3fARB;
  GLEXTERN PFNGLUNIFORM4FARBPROC glUniform4fARB;
  GLEXTERN PFNGLUNIFORM1IARBPROC glUniform1iARB;
  GLEXTERN PFNGLUNIFORM2IARBPROC glUniform2iARB;
  GLEXTERN PFNGLUNIFORM3IARBPROC glUniform3iARB;
  GLEXTERN PFNGLUNIFORM4IARBPROC glUniform4iARB;
  GLEXTERN PFNGLUNIFORM1FVARBPROC glUniform1fvARB;
  GLEXTERN PFNGLUNIFORM2FVARBPROC glUniform2fvARB;
  GLEXTERN PFNGLUNIFORM3FVARBPROC glUniform3fvARB;
  GLEXTERN PFNGLUNIFORM4FVARBPROC glUniform4fvARB;
  GLEXTERN PFNGLUNIFORM1IVARBPROC glUniform1ivARB;
  GLEXTERN PFNGLUNIFORM2IVARBPROC glUniform2ivARB;
  GLEXTERN PFNGLUNIFORM3IVARBPROC glUniform3ivARB;
  GLEXTERN PFNGLUNIFORM4IVARBPROC glUniform4ivARB;
  GLEXTERN PFNGLUNIFORMMATRIX2FVARBPROC glUniformMatrix2fvARB;
  GLEXTERN PFNGLUNIFORMMATRIX3FVARBPROC glUniformMatrix3fvARB;
  GLEXTERN PFNGLUNIFORMMATRIX4FVARBPROC glUniformMatrix4fvARB;
  GLEXTERN PFNGLGETOBJECTPARAMETERFVARBPROC glGetObjectParameterfvARB;
  GLEXTERN PFNGLGETOBJECTPARAMETERIVARBPROC glGetObjectParameterivARB;
  GLEXTERN PFNGLGETINFOLOGARBPROC glGetInfoLogARB;
  GLEXTERN PFNGLGETATTACHEDOBJECTSARBPROC glGetAttachedObjectsARB;
  GLEXTERN PFNGLGETUNIFORMLOCATIONARBPROC glGetUniformLocationARB;
  GLEXTERN PFNGLGETACTIVEUNIFORMARBPROC glGetActiveUniformARB;
  GLEXTERN PFNGLGETUNIFORMFVARBPROC glGetUniformfvARB;
  GLEXTERN PFNGLGETUNIFORMIVARBPROC glGetUniformivARB;
  GLEXTERN PFNGLGETSHADERSOURCEARBPROC glGetShaderSourceARB;
#endif

#ifdef GL_ARB_vertex_shader
  GLEXTERN PFNGLBINDATTRIBLOCATIONARBPROC glBindAttribLocationARB;
  GLEXTERN PFNGLGETACTIVEATTRIBARBPROC glGetActiveAttribARB;
  GLEXTERN PFNGLGETATTRIBLOCATIONARBPROC glGetAttribLocationARB;
#endif

#ifdef GL_ARB_vertex_program
  GLEXTERN PFNGLVERTEXATTRIB1DARBPROC glVertexAttrib1dARB;
  GLEXTERN PFNGLVERTEXATTRIB1DVARBPROC glVertexAttrib1dvARB;
  GLEXTERN PFNGLVERTEXATTRIB1FARBPROC glVertexAttrib1fARB;
  GLEXTERN PFNGLVERTEXATTRIB1FVARBPROC glVertexAttrib1fvARB;
  GLEXTERN PFNGLVERTEXATTRIB1SARBPROC glVertexAttrib1sARB;
  GLEXTERN PFNGLVERTEXATTRIB1SVARBPROC glVertexAttrib1svARB;
  GLEXTERN PFNGLVERTEXATTRIB2DARBPROC glVertexAttrib2dARB;
  GLEXTERN PFNGLVERTEXATTRIB2DVARBPROC glVertexAttrib2dvARB;
  GLEXTERN PFNGLVERTEXATTRIB2FARBPROC glVertexAttrib2fARB;
  GLEXTERN PFNGLVERTEXATTRIB2FVARBPROC glVertexAttrib2fvARB;
  GLEXTERN PFNGLVERTEXATTRIB2SARBPROC glVertexAttrib2sARB;
  GLEXTERN PFNGLVERTEXATTRIB2SVARBPROC glVertexAttrib2svARB;
  GLEXTERN PFNGLVERTEXATTRIB3DARBPROC glVertexAttrib3dARB;
  GLEXTERN PFNGLVERTEXATTRIB3DVARBPROC glVertexAttrib3dvARB;
  GLEXTERN PFNGLVERTEXATTRIB3FARBPROC glVertexAttrib3fARB;
  GLEXTERN PFNGLVERTEXATTRIB3FVARBPROC glVertexAttrib3fvARB;
  GLEXTERN PFNGLVERTEXATTRIB3SARBPROC glVertexAttrib3sARB;
  GLEXTERN PFNGLVERTEXATTRIB3SVARBPROC glVertexAttrib3svARB;
  GLEXTERN PFNGLVERTEXATTRIB4NBVARBPROC glVertexAttrib4NbvARB;
  GLEXTERN PFNGLVERTEXATTRIB4NIVARBPROC glVertexAttrib4NivARB;
  GLEXTERN PFNGLVERTEXATTRIB4NSVARBPROC glVertexAttrib4NsvARB;
  GLEXTERN PFNGLVERTEXATTRIB4NUBARBPROC glVertexAttrib4NubARB;
  GLEXTERN PFNGLVERTEXATTRIB4NUBVARBPROC glVertexAttrib4NubvARB;
  GLEXTERN PFNGLVERTEXATTRIB4NUIVARBPROC glVertexAttrib4NuivARB;
  GLEXTERN PFNGLVERTEXATTRIB4NUSVARBPROC glVertexAttrib4NusvARB;
  GLEXTERN PFNGLVERTEXATTRIB4BVARBPROC glVertexAttrib4bvARB;
  GLEXTERN PFNGLVERTEXATTRIB4DARBPROC glVertexAttrib4dARB;
  GLEXTERN PFNGLVERTEXATTRIB4DVARBPROC glVertexAttrib4dvARB;
  GLEXTERN PFNGLVERTEXATTRIB4FARBPROC glVertexAttrib4fARB;
  GLEXTERN PFNGLVERTEXATTRIB4FVARBPROC glVertexAttrib4fvARB;
  GLEXTERN PFNGLVERTEXATTRIB4IVARBPROC glVertexAttrib4ivARB;
  GLEXTERN PFNGLVERTEXATTRIB4SARBPROC glVertexAttrib4sARB;
  GLEXTERN PFNGLVERTEXATTRIB4SVARBPROC glVertexAttrib4svARB;
  GLEXTERN PFNGLVERTEXATTRIB4UBVARBPROC glVertexAttrib4ubvARB;
  GLEXTERN PFNGLVERTEXATTRIB4UIVARBPROC glVertexAttrib4uivARB;
  GLEXTERN PFNGLVERTEXATTRIB4USVARBPROC glVertexAttrib4usvARB;
  GLEXTERN PFNGLVERTEXATTRIBPOINTERARBPROC glVertexAttribPointerARB;
  GLEXTERN PFNGLENABLEVERTEXATTRIBARRAYARBPROC glEnableVertexAttribArrayARB;
  GLEXTERN PFNGLDISABLEVERTEXATTRIBARRAYARBPROC glDisableVertexAttribArrayARB;
  GLEXTERN PFNGLPROGRAMSTRINGARBPROC glProgramStringARB;
  GLEXTERN PFNGLBINDPROGRAMARBPROC glBindProgramARB;
  GLEXTERN PFNGLDELETEPROGRAMSARBPROC glDeleteProgramsARB;
  GLEXTERN PFNGLGENPROGRAMSARBPROC glGenProgramsARB;
  GLEXTERN PFNGLPROGRAMENVPARAMETER4DARBPROC glProgramEnvParameter4dARB;
  GLEXTERN PFNGLPROGRAMENVPARAMETER4DVARBPROC glProgramEnvParameter4dvARB;
  GLEXTERN PFNGLPROGRAMENVPARAMETER4FARBPROC glProgramEnvParameter4fARB;
  GLEXTERN PFNGLPROGRAMENVPARAMETER4FVARBPROC glProgramEnvParameter4fvARB;
  GLEXTERN PFNGLPROGRAMLOCALPARAMETER4DARBPROC glProgramLocalParameter4dARB;
  GLEXTERN PFNGLPROGRAMLOCALPARAMETER4DVARBPROC glProgramLocalParameter4dvARB;
  GLEXTERN PFNGLPROGRAMLOCALPARAMETER4FARBPROC glProgramLocalParameter4fARB;
  GLEXTERN PFNGLPROGRAMLOCALPARAMETER4FVARBPROC glProgramLocalParameter4fvARB;
  GLEXTERN PFNGLGETPROGRAMENVPARAMETERDVARBPROC glGetProgramEnvParameterdvARB;
  GLEXTERN PFNGLGETPROGRAMENVPARAMETERFVARBPROC glGetProgramEnvParameterfvARB;
  GLEXTERN PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC glGetProgramLocalParameterdvARB;
  GLEXTERN PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC glGetProgramLocalParameterfvARB;
  GLEXTERN PFNGLGETPROGRAMIVARBPROC glGetProgramivARB;
  GLEXTERN PFNGLGETPROGRAMSTRINGARBPROC glGetProgramStringARB;
  GLEXTERN PFNGLGETVERTEXATTRIBDVARBPROC glGetVertexAttribdvARB;
  GLEXTERN PFNGLGETVERTEXATTRIBFVARBPROC glGetVertexAttribfvARB;
  GLEXTERN PFNGLGETVERTEXATTRIBIVARBPROC glGetVertexAttribivARB;
  GLEXTERN PFNGLGETVERTEXATTRIBPOINTERVARBPROC glGetVertexAttribPointervARB;
  GLEXTERN PFNGLISPROGRAMARBPROC glIsProgramARB;
#endif

#ifdef GL_ARB_multitexture
  GLEXTERN PFNGLACTIVETEXTUREARBPROC glActiveTextureARB;
  GLEXTERN PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB;
  GLEXTERN PFNGLMULTITEXCOORD1DARBPROC glMultiTexCoord1dARB;
  GLEXTERN PFNGLMULTITEXCOORD1DVARBPROC glMultiTexCoord1dvARB;
  GLEXTERN PFNGLMULTITEXCOORD1FARBPROC glMultiTexCoord1fARB;
  GLEXTERN PFNGLMULTITEXCOORD1FVARBPROC glMultiTexCoord1fvARB;
  GLEXTERN PFNGLMULTITEXCOORD1IARBPROC glMultiTexCoord1iARB;
  GLEXTERN PFNGLMULTITEXCOORD1IVARBPROC glMultiTexCoord1ivARB;
  GLEXTERN PFNGLMULTITEXCOORD1SARBPROC glMultiTexCoord1sARB;
  GLEXTERN PFNGLMULTITEXCOORD1SVARBPROC glMultiTexCoord1svARB;
  GLEXTERN PFNGLMULTITEXCOORD2DARBPROC glMultiTexCoord2dARB;
  GLEXTERN PFNGLMULTITEXCOORD2DVARBPROC glMultiTexCoord2dvARB;
  GLEXTERN PFNGLMULTITEXCOORD2FARBPROC glMultiTexCoord2fARB;
  GLEXTERN PFNGLMULTITEXCOORD2FVARBPROC glMultiTexCoord2fvARB;
  GLEXTERN PFNGLMULTITEXCOORD2IARBPROC glMultiTexCoord2iARB;
  GLEXTERN PFNGLMULTITEXCOORD2IVARBPROC glMultiTexCoord2ivARB;
  GLEXTERN PFNGLMULTITEXCOORD2SARBPROC glMultiTexCoord2sARB;
  GLEXTERN PFNGLMULTITEXCOORD2SVARBPROC glMultiTexCoord2svARB;
  GLEXTERN PFNGLMULTITEXCOORD3DARBPROC glMultiTexCoord3dARB;
  GLEXTERN PFNGLMULTITEXCOORD3DVARBPROC glMultiTexCoord3dvARB;
  GLEXTERN PFNGLMULTITEXCOORD3FARBPROC glMultiTexCoord3fARB;
  GLEXTERN PFNGLMULTITEXCOORD3FVARBPROC glMultiTexCoord3fvARB;
  GLEXTERN PFNGLMULTITEXCOORD3IARBPROC glMultiTexCoord3iARB;
  GLEXTERN PFNGLMULTITEXCOORD3IVARBPROC glMultiTexCoord3ivARB;
  GLEXTERN PFNGLMULTITEXCOORD3SARBPROC glMultiTexCoord3sARB;
  GLEXTERN PFNGLMULTITEXCOORD3SVARBPROC glMultiTexCoord3svARB;
  GLEXTERN PFNGLMULTITEXCOORD4DARBPROC glMultiTexCoord4dARB;
  GLEXTERN PFNGLMULTITEXCOORD4DVARBPROC glMultiTexCoord4dvARB;
  GLEXTERN PFNGLMULTITEXCOORD4FARBPROC glMultiTexCoord4fARB;
  GLEXTERN PFNGLMULTITEXCOORD4FVARBPROC glMultiTexCoord4fvARB;
  GLEXTERN PFNGLMULTITEXCOORD4IARBPROC glMultiTexCoord4iARB;
  GLEXTERN PFNGLMULTITEXCOORD4IVARBPROC glMultiTexCoord4ivARB;
  GLEXTERN PFNGLMULTITEXCOORD4SARBPROC glMultiTexCoord4sARB;
  GLEXTERN PFNGLMULTITEXCOORD4SVARBPROC glMultiTexCoord4svARB;
#endif

#ifdef GL_ATI_draw_buffers
  GLEXTERN PFNGLDRAWBUFFERSATIPROC glDrawBuffersATI;
#endif

#ifdef GL_EXT_framebuffer_object
  GLEXTERN PFNGLISRENDERBUFFEREXTPROC glIsRenderbufferEXT;
  GLEXTERN PFNGLBINDRENDERBUFFEREXTPROC glBindRenderbufferEXT;
  GLEXTERN PFNGLDELETERENDERBUFFERSEXTPROC glDeleteRenderbuffersEXT;
  GLEXTERN PFNGLGENRENDERBUFFERSEXTPROC glGenRenderbuffersEXT;
  GLEXTERN PFNGLRENDERBUFFERSTORAGEEXTPROC glRenderbufferStorageEXT;
  GLEXTERN PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC glGetRenderbufferParameterivEXT;
  GLEXTERN PFNGLISFRAMEBUFFEREXTPROC glIsFramebufferEXT;
  GLEXTERN PFNGLBINDFRAMEBUFFEREXTPROC glBindFramebufferEXT;
  GLEXTERN PFNGLDELETEFRAMEBUFFERSEXTPROC glDeleteFramebuffersEXT;
  GLEXTERN PFNGLGENFRAMEBUFFERSEXTPROC glGenFramebuffersEXT;
  GLEXTERN PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC glCheckFramebufferStatusEXT;
  GLEXTERN PFNGLFRAMEBUFFERTEXTURE1DEXTPROC glFramebufferTexture1DEXT;
  GLEXTERN PFNGLFRAMEBUFFERTEXTURE2DEXTPROC glFramebufferTexture2DEXT;
  GLEXTERN PFNGLFRAMEBUFFERTEXTURE3DEXTPROC glFramebufferTexture3DEXT;
  GLEXTERN PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC glFramebufferRenderbufferEXT;
  GLEXTERN PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC glGetFramebufferAttachmentParameterivEXT;
  GLEXTERN PFNGLGENERATEMIPMAPEXTPROC glGenerateMipmapEXT;
#endif 

#ifdef GL_EXT_texture3D
  GLEXTERN PFNGLTEXIMAGE3DEXTPROC glTexImage3DEXT;
  GLEXTERN PFNGLTEXSUBIMAGE3DEXTPROC glTexSubImage3DEXT;
#endif

#undef GLEXTERN

} // end namespace internal
} // end namespace shallows
#endif //_WIN32

#endif // _SHALLOWS_EXT_HPP_
