/**********************************************************************
 * 
 *  File: 
 * 
 *  Created:
 *
 *  Version: $Id: shallowsexcept.hpp,v 1.12 2005/09/26 12:23:05 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *             Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *             Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/
#ifndef _SHALLOWSEXCEPT_HPP_
#define _SHALLOWSEXCEPT_HPP_
#include "shallows/config.h"

#include <stdexcept>
#include <string>
namespace shallows {

/**
 * \brief Base class for invalid argument exceptions in shallows
 */
class  shallows_invalid_argument : public std::invalid_argument {
public:
   shallows_invalid_argument( const std::string& arg ) : invalid_argument( arg ) {};
};

/**
 * \brief Base class for runtime errors exceptions in shallows
 */
class  shallows_runtime_error : public std::runtime_error {
public:
   shallows_runtime_error( const std::string& arg ) : runtime_error( arg ) {};
};

/**
 * \brief Base class for runtime errors exceptions in shallows
 */
class  shallows_logic_error : public std::logic_error {
public:
   shallows_logic_error( const std::string& arg ) : logic_error( arg ) {};
};

   /** \brief Exception class. Thrown if graphics system does not support
    *  required OpenGL extension */
class  missing_extensions : public shallows_runtime_error {
public:
   missing_extensions( const std::string& arg ) : shallows_runtime_error( arg ) {};
};

   /** \brief Exception class. Thrown when missing shallows functionality is 
    *  called. This exception will not be thrown in major versions of
    *  shallows. */
class  not_implemented : public shallows_runtime_error {
public:
        not_implemented( const std::string& arg ) : shallows_runtime_error( arg ) {};
};

   /** \brief Exception class. Thrown when the application tries to open
    *  a missing file */
class file_not_found : public shallows_runtime_error {
public:
        file_not_found( const std::string& arg ) : shallows_runtime_error( arg ) {};
};

   /*
class  no_internal_format : public shallows_runtime_error {
public:
   no_internal_format( const std::string& arg ) : shallows_runtime_error( arg ) {};
};
   */
   /** \brief Exception class. Thrown when a OffScreenBuffer is used or 
    *  validated for completeness, and OpenGL states that it is
    *  incomplete.*/
class  frame_buffer_incomplete : public shallows_runtime_error {
public:
        frame_buffer_incomplete( const std::string& arg, 
                                 unsigned int status ) : 
           shallows_runtime_error( arg ), status_(status) {};
   /** Returns the status given by glCheckFramebufferStatus */
   unsigned int status() {
      return status_;
   }
private:
   unsigned int status_;
};

   /** \brief Exception class. Thrown when a Program fails to compile. */
class  compile_error : public shallows_logic_error {
public:
        compile_error( const std::string& arg, const std::string& log, const std::string& source ) : 
        shallows_logic_error( arg ),
        log_( log ),
        source_ ( source )
        {};
        
        virtual 
    ~compile_error() throw() {};
    
   /** Returns the error log (provided by graphics system) */
        std::string log() const throw() {
                return log_;
        };
        
   /** Returns the shader source of the shader that did not compile */
        std::string source() const throw() {
                return source_;
        };
        
private:
        std::string log_;
        std::string source_;
};
} // namespace shallows
#endif //_SHALLOWSEXCEPT_HPP_
