/**********************************************************************
 * 
 *  File: imageCreation.hpp
 * 
 *  Created:
 *
 *  Version: $Id: imageCreation.hpp,v 1.4 2005/09/18 21:06:30 josse77 Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/
#ifndef _IMAGECREATION_HPP_
#define _IMAGECREATION_HPP_

#include "shallows/config.h"
#include "shallows/utils/Image.hpp"
#include <string>
#include <boost/shared_ptr.hpp>

namespace shallows {
namespace utils {

/** Get a float image from an image file. */
DllExport 
boost::shared_ptr<Image<float> >
getFloatImage( std::string filename, unsigned int channels = 4 );

/** Get an int image from an image file. */
DllExport 
boost::shared_ptr<Image<int> >
getIntImage( std::string filename, unsigned int channels = 4 );

/** Get a char image from an image file. */
DllExport 
boost::shared_ptr<Image<char> >
getCharImage( std::string filename, unsigned int channels = 4 );

/** Write a float image to an image file. */
DllExport 
void
writeImageToFile( std::string filename, const Image<float>& image );

/** Write a char image to an image file. */
DllExport 
void
writeImageToFile( std::string filename, const Image<char>& image );

/** Write an int image to an image file. */
DllExport 
void
writeImageToFile( std::string filename, const Image<int>& image );

}; // namespace utils
}; // namespace shallows

#endif //_IMAGECREATION_HPP_
