/**********************************************************************
 * 
 *  File: readBuffer.hpp
 * 
 *  Created:
 *
 *  Version: $Id: readBuffer.hpp,v 1.7 2005/11/25 11:18:15 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef _READBUFFER_HPP_
#define _READBUFFER_HPP_

#include "shallows/config.h"
#include <vector>
#include <boost/shared_ptr.hpp>

namespace shallows {
class FrameBuffer;
class RenderTarget;
namespace utils {

/** Reads the content of the rendertarget. The result is read into 
 * a vector of floats.
 * \param fb The framebuffer to bind the render target.
 * \param rt The render target to be read.
 * \param components The number of components in the rendertarget
 * \param width The width to read (should not be greater than the width of the 
 * rendertarget
 * \param height The height to read (should not be greater than the height 
 * of the rendertarget
 * \param pixels This vector is resized inside the function before it is 
 * filled with the result
 */
 DllExport
void readFloatBuffer(shallows::FrameBuffer &fb, 
		     boost::shared_ptr<shallows::RenderTarget> rt,
		     int components, unsigned int width, unsigned int height,
		     std::vector<float> &pixels);

/** Reads the content of the rendertarget. The result is read into 
 * a vector of unsigned char.
 * \param fb The framebuffer to bind the render target.
 * \param rt The render target to be read.
 * \param components The number of components in the rendertarget
 * \param width The width to read (should not be greater than the width of the 
 * rendertarget
 * \param height The height to read (should not be greater than the height 
 * of the rendertarget
 * \param pixels This vector is resized inside the function before it is 
 * filled with the result
 */
DllExport
void 
readUnsignedCharBuffer(FrameBuffer &fb, 
		       boost::shared_ptr<shallows::RenderTarget> rt,
		       int components, unsigned int width, unsigned int height,
		       std::vector<unsigned char> &pixels);
} // namespace utils
} // namespace shallows

#endif // _READBUFFER_HPP_
