/**********************************************************************
 * 
 *  File: textureCreation.hpp
 * 
 *  Created: 11. July 2005
 *
 *  Version: $Id: textureCreation.hpp,v 1.7 2005/10/26 12:35:29 dyken Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	         Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	         Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef _SHALLOWS_TEXTURECREATION_HPP_
#define _SHALLOWS_TEXTURECREATION_HPP_

/** \file textureCreation.hpp This file contains utility functions to easily generate a Texture2D
 *  from a blob of data. This is useful for importing data fra various 
 *  file readers etc.
 */
#include "shallows/config.h"
#include <boost/shared_ptr.hpp>

namespace shallows {
class Texture2D;
class Texture1D;
class Texture3D;
namespace utils{ 

/** Create a float Texture2D of the gives size from a pointer. */
DllExport
boost::shared_ptr<Texture2D> 
createFloatTexture2D(unsigned int width, 
						     unsigned int height, 
						     unsigned int components,
						     const float *pixels);

/** Create a float Texture1D of the gives size from a pointer. */
DllExport
boost::shared_ptr<Texture1D> 
createFloatTexture1D(unsigned int width, 
						     unsigned int components,
						     const float *pixels);

/** Create a float Texture2D rectangle of the given size from a pointer. */
DllExport
boost::shared_ptr<Texture2D> 
createFloatTextureRectangle(unsigned int width, 
				 unsigned int height, 
				 unsigned int components,
				 const float *pixels);
                 
/** Create an unsigned char Texture2D of the given size from a pointer. */
DllExport   
boost::shared_ptr<Texture2D> 
createUnsignedByteTexture2D(unsigned int width, 
			       unsigned int height,
			       unsigned int components,
			       const unsigned char *pixels);

/** Create an unsigned char Texture2D rectangle of the given size from a pointer. */
DllExport
boost::shared_ptr<Texture2D> 
createUnsignedByteTextureRectangle(unsigned int width, 
			       unsigned int height,
			       unsigned int components,
			       const unsigned char *pixels);

/** Create a float Texture3D of a given size from a pointer. */
DllExport
boost::shared_ptr<Texture3D>
createFloatTexture3D(unsigned int width, 
					 unsigned int height,
					 unsigned int depth,
					 unsigned int components,
					 const float *pixels);

} // namespace shallows::utils
} // namespace shallows

#endif // _SHALLOWS_TEXTURECREATION_HPP_
