/**********************************************************************
 * 
 *  File: 
 * 
 *  Created:
 *
 *  Version: $Id: Timer.hpp,v 1.4 2005/09/25 21:34:26 josse77 Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *       Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *       Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifndef TIMER_HPP_
#define TIMER_HPP_

#include "shallows/config.h"

#ifdef UNIX
#include <sys/time.h>
#include <unistd.h>
#endif

#ifdef _WIN32
#include <sys/timeb.h>
#include <time.h>
#endif


namespace shallows {
namespace utils {

/** \brief A very basic timer class, suitable for FPS counters etc. 
 *  
 */
class Timer
{
    
public:
	Timer(): start_time_(getCurrentTime() ) {};
    
    /** Report the elapsed time in seconds (it will return a double,
     *  so the fractional part is subsecond part). */
    double elapsed() const { return getCurrentTime() - start_time_; };
    
    /** Restart the timer. */
    void restart() { start_time_ = getCurrentTime(); };
    
    /** Return the current time as number of elapsed seconds of this day. */
    double static getCurrentTime() {

#ifdef UNIX
  struct timeval tv;
  struct timezone tz;
  gettimeofday(&tv, &tz);
  return tv.tv_sec+tv.tv_usec*1e-6;
#endif
 
#ifdef _WIN32
  struct _timeb ts;
  _ftime64_s(&ts);
  return (ts.time + 0.001*ts.millitm);
#endif
    };

private:
    double start_time_;
    
};
    } // namespace utils
} // namespace timer
#endif /*TIMER_HPP_*/
