/**********************************************************************
 * 
 *  File: 
 * 
 *  Created:
 *
 *  Version: $Id: CgProgramImpl.cpp,v 1.6 2005/09/12 14:33:09 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/
#include "shallows/config.h"
#if defined(HAVE_CG) && defined(HAVE_CGGL)
 
 #include "shallows/internal/CgProgramImpl.hpp"
 #include "shallows/internal/CgShader.hpp"
 #include "shallows/internal/GLIncludes.hpp"
 #include "shallows/shallows.hpp"
 #include "shallows/shallowsexcept.hpp"

 namespace shallows {
 namespace internal {


CgProgramImpl::CgProgramImpl( shallows::CgProgram& parent ) :
parent_( parent ) {
}

CgProgramImpl::pCgShader 
CgProgramImpl::initShader( pCgShader shader, internal::Shader::ShaderType type,
	CGcontext context, CGprofile profile, std::string source ) {
	
	if (!source.empty()) {
		shader.reset( new internal::CgShader(type, context, profile) );
		if ( shader->read( source ) ) {
      		cgGLLoadProgram( shader->getShaderID() );
		} else {
      		CGerror LastError = cgGetError();
      		parent_.addToLog( cgGetErrorString(LastError) );
//      		throw shallows::compile_error( "Shader compile error", parent_.getLog(), cgGetLastListing( context ) );
      		throw shallows::compile_error( "Shader compile error", cgGetLastListing( context ), source );
    	}
  } else
    shader.reset( NULL );
    
  CHECK_GL_DEBUG;
  CHECK_CG_DEBUG;
  
  return shader;
}
		
CGparameter
CgProgramImpl::validateTexture( const char* name,
				const Texture &texture) const
{
	CGparameter cgp;
	if (fragmentShader_.get())
	   cgp = cgGetNamedParameter( fragmentShader_->getShaderID(), name);
  	if( cgp == 0 && vertexShader_.get()) 
  		cgp = cgGetNamedParameter( vertexShader_->getShaderID(), name);
	if ( cgp == 0 ){
  		std::string s = "Uniform variable ";
  		s += name;
  		s += " was not found or illegal.";
  		throw std::invalid_argument( s );
	}
  	
  	shallows::Program::NameTypeMap::const_iterator it = parent_.uniforms.find( name );
	assert( it != parent_.uniforms.end() && 
		"Internal Shallows error. The uniform map does not correspond with the Cg runtime." );
	
	std::string sampler_type
	   (cgGetTypeString(cgGetParameterType(cgp)));
	texture.validateSamplerType(sampler_type);

	
	CHECK_GL_DEBUG;
    CHECK_CG_DEBUG;
	return cgp;
}

CGparameter
CgProgramImpl::validateUniform( const char* name ) const {
	
	CGparameter cgp;
	if (fragmentShader_.get())
	   cgp = cgGetNamedParameter( fragmentShader_->getShaderID(), name);
  	if( cgp == 0 && vertexShader_.get()) 
  		cgp = cgGetNamedParameter( vertexShader_->getShaderID(), name);
	if ( cgp == 0 ){
  		std::string s = "Uniform variable ";
  		s += name;
  		s += " was not found or illegal.";
  		throw std::invalid_argument( s );
	}
  	
  	shallows::Program::NameTypeMap::const_iterator it = parent_.uniforms.find( name );
	assert( it != parent_.uniforms.end() && 
		"Internal Shallows error. The uniform map does not correspond with the Cg runtime." );
	
	std::string type = it->second;
	if ( !type.find( "sampler" ) != std::string::npos ) {
		std::string s = "Uniform variable ";
		s += name;
		s += " is  a sampler (texture). It can not be set this way. ";
		throw std::invalid_argument( s );
	}
	
	CHECK_GL_DEBUG;
    CHECK_CG_DEBUG;
	return cgp;
}
		
CGparameter
CgProgramImpl::validateAttrib( const char *name ) const {
	CGparameter cgp = cgGetNamedParameter( fragmentShader_->getShaderID(), name);
  	if( cgp==0 ) 
  		cgp = cgGetNamedParameter( vertexShader_->getShaderID(), name);
  	if ( cgp == 0 ) {
  		std::string s = "Attribute variable ";
  		s += name;
  		s += " was not found or illegal.";
  		throw std::invalid_argument( s );
	}
	CHECK_GL_DEBUG;
    CHECK_CG_DEBUG;
    return cgp;
}

void
CgProgramImpl::buildUniformsMap()  {
    parent_.uniformsNameToType_.clear();
    shallows::Program::NameTypeMap vertMap, fragMap;
    if ( vertexShader_.get() != NULL )
        vertMap = vertexShader_->getUniformsMap();
    
    if ( fragmentShader_.get() != NULL )     
        fragMap = fragmentShader_->getUniformsMap();
    
    parent_.uniformsNameToType_.insert( vertMap.begin(), vertMap.end() );
    parent_.uniformsNameToType_.insert( fragMap.begin(), fragMap.end() );    
}

 } // namespace internal
 } // namespace shallows
 #endif // HAVE CG
