/**********************************************************************
 * 
 *  File: 
 * 
 *  Created:
 *
 *  Version: $Id: CgShader.cpp,v 1.7 2005/09/15 13:13:12 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/
#include "shallows/config.h"
#if defined(HAVE_CG) && defined(HAVE_CGGL)
#ifdef _WIN32
  #include <windows.h>
#endif

#include "shallows/shallows.hpp"
#include "shallows/internal/CgShader.hpp"
#include "shallows/internal/GLIncludes.hpp"

namespace shallows {
namespace internal {
	
using namespace std;

CgShader::CgShader() 
{

}

CgShader::CgShader(ShaderType shaderType, CGcontext context, CGprofile profile) : 	
	Shader(shaderType), 
	context_(context), 
	profile_(profile) 
{
}

CgShader::~CgShader()
{

}

bool 
CgShader::read(std::string shaderText)
{
  const char *cstr=shaderText.c_str();

  shader_ = cgCreateProgram(context_, CG_SOURCE, cstr, profile_, NULL, NULL);
  
  if (shader_==0)
     return false;
  CHECK_GL_DEBUG;
  CHECK_CG_DEBUG;
  return true;
  
}

static void RecurseProgramParametersEnableTexures(CGparameter parameter) 
{ 
  int arraySize;
  if (parameter == 0) return; 
  do 
    { 
      switch(cgGetParameterType(parameter)) 
	{ 
	case CG_STRUCT: 
	  RecurseProgramParametersEnableTexures( 
	    cgGetFirstStructParameter(parameter)); 
	  break; 
	case CG_ARRAY: 
	  arraySize = cgGetArraySize(parameter, 0); 
	  for (int i = 0; i < arraySize; ++i) 
	    RecurseProgramParametersEnableTexures( 
	    cgGetArrayParameter(parameter, i)); 
	  break;
	case CG_SAMPLER2D:
    {
	    cgGLEnableTextureParameter(parameter);
//      GLenum l = cgGLGetTextureEnum(parameter);
//      int q=0;
	    break;
    }
	case CG_SAMPLER1D:
    {
      cgGLEnableTextureParameter(parameter);
//      GLenum k = cgGLGetTextureEnum(parameter);
//      int p=0;
	    break;
    }
	case CG_SAMPLERRECT:
    {
      cgGLEnableTextureParameter(parameter);
	    break;
    }
	default: /* Here is the code that handles the parameter */ 
	  break; 
	} 
    } while((parameter = cgGetNextParameter(parameter))!= 0); 
}

void
CgShader::enable()
{
   CGparameter cgp = cgGetFirstParameter(shader_, CG_PROGRAM);
   RecurseProgramParametersEnableTexures(cgp);

   cgGLEnableProfile(profile_);
   cgGLBindProgram(shader_);
}

CGprogram 
CgShader::getShaderID()
{
  return shader_;
}

shallows::Program::NameTypeMap 
CgShader::getUniformsMap() {
    
    struct RecurseParams {
        void operator()( shallows::Program::NameTypeMap& map, CGparameter param ) {
            if (!param)
                return;
            
            do {
                switch ( cgGetParameterType(param) ) {
                    case CG_STRUCT:
                        RecurseParams()( map, cgGetFirstStructParameter( param ) );
                        break;
                    case CG_ARRAY:  {
                        int size = cgGetArraySize( param, 0 );
                        for ( int i = 0; i < size; ++i ) 
                            RecurseParams()( map, cgGetArrayParameter( param, i ) );
                    }
                    break;
                    
                    default:
                        if ( cgGetParameterVariability( param ) == CG_UNIFORM )
                            map[cgGetParameterName(param)] = cgGetTypeString(cgGetParameterType(param));
                }
            }  while((param = cgGetNextParameter(param)) != 0);
        }
    };
    shallows::Program::NameTypeMap map;
    
    CGparameter cgp = cgGetFirstParameter( shader_, CG_PROGRAM );
    
    RecurseParams()( map, cgp );
    
    return map;
}


} // namespace internal
} // namespace shallows
#endif // HAVE_CG
