/**********************************************************************
 * 
 *  File: GLShader.cpp
 * 
 *  Created: 20050621
 *
 *  Version: $Id: GLShader.cpp,v 1.3 2005/06/24 10:40:16 johans Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#ifdef _WIN32
  #include <windows.h>
#endif

#include <cassert>

#include "shallows/internal/GLShader.hpp"
#include "shallows/shallows.hpp"
#include "shallows/internal/GLIncludes.hpp"


namespace shallows {
namespace internal {

using std::string;
	
GLShader::GLShader() :
  shader_( 0 )
{
}

GLShader::GLShader(ShaderType shaderType) : Shader(shaderType)
{
  if(shaderType_ == VertexShader)
  {
	  shader_ = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
  } 
  else if ( shaderType_ == FragmentShader )
  {
	  shader_ = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
  } 
  else 
  	assert( false && "Invalid shaderType given" );
  	
  CHECK_GL_DEBUG;
}

GLShader::~GLShader()
{
	if ( shader_ != 0 )
		glDeleteObjectARB( shader_ );
	// Do not throw in d'tors.
	// CHECK_GL_DEBUG;
}

bool GLShader::read(const std::string shaderText)
{
  int result;
  const char *cstr=shaderText.c_str();
 	glShaderSourceARB(shader_, 1, &cstr, NULL);
	glCompileShaderARB(shader_);
	glGetObjectParameterivARB(shader_, GL_OBJECT_COMPILE_STATUS_ARB, &result);
  
  CHECK_GL_DEBUG;
  return result != 0;
}

GLhandleARB GLShader::getShaderID() const 
{
  return shader_;
}

} // namespace internal
} // namespace shallows
