/**********************************************************************
 * 
 *  File: file
 * 
 *  Created: Jul 13, 2005
 *
 *  Version: $Id: Image.cpp,v 1.6 2005/09/18 21:06:30 josse77 Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/
#include "shallows/utils/Image.hpp"
#include "shallows/internal/GLIncludes.hpp"
#include "shallows/utils/utilsexcept.hpp"

namespace shallows {

namespace utils {

unsigned int
channelsToGLenum( const unsigned int channels ) {
    switch ( channels ) {
        case 4:
            return GL_RGBA;
        case 3:
            return GL_RGB;
        case 2:
            return GL_LUMINANCE_ALPHA; 
        case 1:
            return GL_LUMINANCE;
        default:
            throw not_supported( "OpenGL only support up to 4 channels. " );
    }
}

template<>
unsigned int
type2GLExternalFormat<double>() {
    return GL_DOUBLE;
}

template<>
unsigned int
type2GLExternalFormat<float>() {
    return GL_FLOAT;
}

template<>
unsigned int
type2GLExternalFormat<int>() {
    return GL_INT;
}

template<>
unsigned int
type2GLExternalFormat<unsigned int>() {
    return GL_UNSIGNED_INT;
}

template<>
unsigned int
type2GLExternalFormat<char>() {
    return GL_UNSIGNED_BYTE;
}


template<>
unsigned int
type2GLExternalFormat<unsigned char>() {
    return GL_UNSIGNED_BYTE;
}

} // namespace utils

} // namespace shallows
