/**********************************************************************
 * 
 *  File: RenderBuffer.cpp
 * 
 *  Created:
 *
 *  Version: $Id: RenderBuffer.cpp,v 1.4 2005/11/25 11:18:16 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#include "shallows/RenderBuffer.hpp"
#include "shallows/shallows.hpp"
#include "shallows/shallowsexcept.hpp"
#include "shallows/internal/GLIncludes.hpp"

using namespace shallows::internal;

namespace shallows 
{

RenderBuffer::RenderBuffer(int internalFormat, unsigned int width,
			   unsigned int height)
   :   internalFormat_(internalFormat), width_(width), height_(height),
       bufId_(0)
{
   glGenRenderbuffersEXT(1, &bufId_);
   glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, bufId_);
   glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT,
			    internalFormat, width_, height_);
  try 
  {
    CHECK_GL;
  } 
  catch ( ... )
  {
    if (bufId_!=0)
      glDeleteRenderbuffersEXT(1, &bufId_);
    bufId_=0;
    throw;
  }
}

RenderBuffer::~RenderBuffer() 
{ 
   if (bufId_!=0)
      glDeleteRenderbuffersEXT(1, &bufId_);
}


void RenderBuffer::bindToCurrentFrameBuffer(int colAttachment)
{
   if (colAttachment<0)
      colAttachment=prevColAttachment_;
   else
      prevColAttachment_=colAttachment;

   glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, bufId_);
   glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT,
				GL_COLOR_ATTACHMENT0_EXT+colAttachment,
				GL_RENDERBUFFER_EXT, bufId_);
    CHECK_GL;
}

void RenderBuffer::bindAsDepthToCurrentFrameBuffer()
{
   glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, bufId_);
   glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT,
				GL_DEPTH_ATTACHMENT_EXT,
				GL_RENDERBUFFER_EXT, bufId_);
    CHECK_GL;
}

unsigned int RenderBuffer::getLastBufferEnum() const
{
   return GL_COLOR_ATTACHMENT0_EXT+prevColAttachment_;
}

unsigned int RenderBuffer::getWidth() const
{
   return width_;
}

unsigned int RenderBuffer::getHeight() const
{
   return height_;

}

int RenderBuffer::getInternalFormat() const
{
   return internalFormat_;
}

void RenderBuffer::attachToCurrentFrameBuffer(int colAttachment)
{
   glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, bufId_);
   glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT,
				GL_COLOR_ATTACHMENT0_EXT+colAttachment,
				GL_RENDERBUFFER_EXT, bufId_);
    CHECK_GL;
}

} // namespace shallows 

