/**********************************************************************
 * 
 *  File: RenderTexture2D.cpp
 * 
 *  Created:
 *
 *  Version: $Id: RenderTexture2D.cpp,v 1.10 2005/11/25 11:18:16 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#include "shallows/RenderTexture2D.hpp"
#include "shallows/shallowsexcept.hpp"
#include "shallows/internal/GLIncludes.hpp"
#include "shallows/shallows.hpp"

using namespace shallows::internal;

namespace shallows 
{

RenderTexture2D::RenderTexture2D(texture_ptr tex)
   :   texture_(tex)
{
   if (tex.get()==0)
   {
      throw std::invalid_argument("Initialized RenderTexture2D with empty texture");
   }
}

RenderTexture2D::~RenderTexture2D() { ; }


void RenderTexture2D::bindToCurrentFrameBuffer(int colAttachment)
{
   if (colAttachment<0)
      colAttachment=prevColAttachment_;
   else
      prevColAttachment_=colAttachment;
   if (texture_->isRectangular())
   {
      glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, 
				GL_COLOR_ATTACHMENT0_EXT+colAttachment, 
				GL_TEXTURE_RECTANGLE_ARB, 
				texture_->getTextureID(), 0);
   } else
   {
      glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, 
				GL_COLOR_ATTACHMENT0_EXT+colAttachment, 
				GL_TEXTURE_2D, texture_->getTextureID(), 0);
   }
}


void RenderTexture2D::bindAsDepthToCurrentFrameBuffer()
{
   GLenum target = (texture_->isRectangular()) ? GL_TEXTURE_RECTANGLE_ARB:
      GL_TEXTURE_2D;

   glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, 
			     GL_DEPTH_ATTACHMENT_EXT,
			     target, 
			     texture_->getTextureID(), 0);
   CHECK_GL;
}


unsigned int RenderTexture2D::getLastBufferEnum() const
{
   return GL_COLOR_ATTACHMENT0_EXT+prevColAttachment_;
}

unsigned int RenderTexture2D::getWidth() const
{
   if (texture_.get() == 0)
   {
      return 0; // or throw?
   }
   return texture_->getWidth();
}

unsigned int RenderTexture2D::getHeight() const
{
   if (texture_.get() == 0)
   {
      return 0; // or throw?
   }
   return texture_->getHeight();
}

int RenderTexture2D::getInternalFormat() const
{
   if (texture_.get() == 0)
   {
      return 0; // or throw?
   }
   return texture_->getInternalFormat();
}

RenderTexture2D::texture_ptr RenderTexture2D::getTexture() const
{
   return texture_;
}

void RenderTexture2D::attachToCurrentFrameBuffer(int colAttachment)
{
   GLenum target = (texture_->isRectangular()) ? GL_TEXTURE_RECTANGLE_ARB:
      GL_TEXTURE_2D;

   glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, 
			     GL_COLOR_ATTACHMENT0_EXT+colAttachment, 
			     target, 
			     texture_->getTextureID(), 0);
   CHECK_GL;
}

} // namespace shallows 
