/**********************************************************************
 * 
 *  File: 
 * 
 *  Created:
 *
 *  Version: $Id: ShallowsExt.cpp,v 1.11 2005/12/06 14:00:35 trondrh Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	         Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	         Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

//#include "shallows/internal/GLIncludes.hpp"



#include <string.h>
#include <stdio.h>
#ifndef _WIN32
namespace shallows 
{
namespace internal
{
bool initShaderExtensions()
{
return true;
}
}
}
#else
#include <windows.h>
#include <GL/gl.h>

#define INITGLEXT
  #include "shallows/internal/ShallowsExt.hpp"
#undef INITGLEXT
//#include "wglext.h"

namespace shallows 
{
namespace internal
{
  bool initShaderExtensions()
  {

	/*****************************************************************************/
	/*	Check if extension is supported                                           */
	/*****************************************************************************/

  if(!internal::isShaderExtensionSupported("GL_ARB_shader_objects"))          return false;
  if(!internal::isShaderExtensionSupported("GL_ARB_fragment_shader"))         return false;
  if(!internal::isShaderExtensionSupported("GL_ARB_vertex_shader"))           return false;
  if(!internal::isShaderExtensionSupported("GL_ARB_shading_language_100"))    return false;
  if(!internal::isShaderExtensionSupported("GL_ARB_vertex_program"))          return false;
  if(!internal::isShaderExtensionSupported("GL_ARB_fragment_program"))        return false;
  if(!internal::isShaderExtensionSupported("GL_ARB_multitexture"))            return false;
  if(!internal::isShaderExtensionSupported("GL_ATI_draw_buffers"))            return false;
  if(!internal::isShaderExtensionSupported("GL_ATI_texture_float"))           return false;
  if(!internal::isShaderExtensionSupported("GL_EXT_framebuffer_object"))      return false;

  /*****************************************************************************/
  /*	Initialize extensions. Only extensions that includes new function call   */
  /*	entry points needs to be initialized                                     */
  /*****************************************************************************/
	
  if(!internal::initShaderExtension("GL_VERSION_1_2"))                        return false;
  if(!internal::initShaderExtension("GL_VERSION_2_0"))                        return false;
  if(!internal::initShaderExtension("GL_ARB_shader_objects"))                 return false;  
  if(!internal::initShaderExtension("GL_ARB_vertex_shader"))                  return false;  
  if(!internal::initShaderExtension("GL_ARB_vertex_program"))                 return false;  
  if(!internal::initShaderExtension("GL_ARB_multitexture"))                   return false;  
  if(!internal::initShaderExtension("GL_ATI_draw_buffers"))                   return false;
  if(!internal::initShaderExtension("GL_EXT_framebuffer_object"))             return false;


  // Everything OK
	return true;
}

  


static bool generateShaderExtension(const char* extension)
{
	if(extension == NULL)
	{
		return false;
	}

  #ifdef GL_VERSION_2_0
  else if(0 == strcmp(extension, "GL_VERSION_2_0"))
  {
	  glBlendEquationSeparate = (PFNGLBLENDEQUATIONSEPARATEPROC)wglGetProcAddress("glBlendEquationSeparate");
	  if(NULL == glBlendEquationSeparate) return false;

	  glDrawBuffers = (PFNGLDRAWBUFFERSPROC)wglGetProcAddress("glDrawBuffers");
	  if(NULL == glDrawBuffers) return false;

	  glStencilOpSeparate = (PFNGLSTENCILOPSEPARATEPROC)wglGetProcAddress("glStencilOpSeparate");
	  if(NULL == glStencilOpSeparate) return false;

	  glStencilFuncSeparate = (PFNGLSTENCILFUNCSEPARATEPROC)wglGetProcAddress("glStencilFuncSeparate");
	  if(NULL == glStencilFuncSeparate) return false;

	  glStencilMaskSeparate = (PFNGLSTENCILMASKSEPARATEPROC)wglGetProcAddress("glStencilMaskSeparate");
	  if(NULL == glStencilMaskSeparate) return false;

	  glAttachShader = (PFNGLATTACHSHADERPROC)wglGetProcAddress("glAttachShader");
	  if(NULL == glAttachShader) return false;

	  glBindAttribLocation = (PFNGLBINDATTRIBLOCATIONPROC)wglGetProcAddress("glBindAttribLocation");
	  if(NULL == glBindAttribLocation) return false;

	  glCompileShader = (PFNGLCOMPILESHADERPROC)wglGetProcAddress("glCompileShader");
	  if(NULL == glCompileShader) return false;

	  glCreateProgram = (PFNGLCREATEPROGRAMPROC)wglGetProcAddress("glCreateProgram");
	  if(NULL == glCreateProgram) return false;

	  glCreateShader = (PFNGLCREATESHADERPROC)wglGetProcAddress("glCreateShader");
	  if(NULL == glCreateShader) return false;

	  glDeleteProgram = (PFNGLDELETEPROGRAMPROC)wglGetProcAddress("glDeleteProgram");
	  if(NULL == glDeleteProgram) return false;

	  glDeleteShader = (PFNGLDELETESHADERPROC)wglGetProcAddress("glDeleteShader");
	  if(NULL == glDeleteShader) return false;

	  glDetachShader = (PFNGLDETACHSHADERPROC)wglGetProcAddress("glDetachShader");
	  if(NULL == glDetachShader) return false;

	  glDisableVertexAttribArray = (PFNGLDISABLEVERTEXATTRIBARRAYPROC)wglGetProcAddress("glDisableVertexAttribArray");
	  if(NULL == glDisableVertexAttribArray) return false;

	  glEnableVertexAttribArray = (PFNGLENABLEVERTEXATTRIBARRAYPROC)wglGetProcAddress("glEnableVertexAttribArray");
	  if(NULL == glEnableVertexAttribArray) return false;

	  glGetActiveAttrib = (PFNGLGETACTIVEATTRIBPROC)wglGetProcAddress("glGetActiveAttrib");
	  if(NULL == glGetActiveAttrib) return false;

	  glGetActiveUniform = (PFNGLGETACTIVEUNIFORMPROC)wglGetProcAddress("glGetActiveUniform");
	  if(NULL == glGetActiveUniform) return false;

	  glGetAttachedShaders = (PFNGLGETATTACHEDSHADERSPROC)wglGetProcAddress("glGetAttachedShaders");
	  if(NULL == glGetAttachedShaders) return false;

	  glGetAttribLocation = (PFNGLGETATTRIBLOCATIONPROC)wglGetProcAddress("glGetAttribLocation");
	  if(NULL == glGetAttribLocation) return false;

	  glGetProgramiv = (PFNGLGETPROGRAMIVPROC)wglGetProcAddress("glGetProgramiv");
	  if(NULL == glGetProgramiv) return false;

	  glGetProgramInfoLog = (PFNGLGETPROGRAMINFOLOGPROC)wglGetProcAddress("glGetProgramInfoLog");
	  if(NULL == glGetProgramInfoLog) return false;

	  glGetShaderiv = (PFNGLGETSHADERIVPROC)wglGetProcAddress("glGetShaderiv");
	  if(NULL == glGetShaderiv) return false;

	  glGetShaderInfoLog = (PFNGLGETSHADERINFOLOGPROC)wglGetProcAddress("glGetShaderInfoLog");
	  if(NULL == glGetShaderInfoLog) return false;

	  glGetShaderSource = (PFNGLGETSHADERSOURCEPROC)wglGetProcAddress("glGetShaderSource");
	  if(NULL == glGetShaderSource) return false;

	  glGetUniformLocation = (PFNGLGETUNIFORMLOCATIONPROC)wglGetProcAddress("glGetUniformLocation");
	  if(NULL == glGetUniformLocation) return false;

	  glGetUniformfv = (PFNGLGETUNIFORMFVPROC)wglGetProcAddress("glGetUniformfv");
	  if(NULL == glGetUniformfv) return false;

	  glGetUniformiv = (PFNGLGETUNIFORMIVPROC)wglGetProcAddress("glGetUniformiv");
	  if(NULL == glGetUniformiv) return false;

	  glGetVertexAttribdv = (PFNGLGETVERTEXATTRIBDVPROC)wglGetProcAddress("glGetVertexAttribdv");
	  if(NULL == glGetVertexAttribdv) return false;

	  glGetVertexAttribfv = (PFNGLGETVERTEXATTRIBFVPROC)wglGetProcAddress("glGetVertexAttribfv");
	  if(NULL == glGetVertexAttribfv) return false;

	  glGetVertexAttribiv = (PFNGLGETVERTEXATTRIBIVPROC)wglGetProcAddress("glGetVertexAttribiv");
	  if(NULL == glGetVertexAttribiv) return false;

	  glGetVertexAttribPointerv = (PFNGLGETVERTEXATTRIBPOINTERVPROC)wglGetProcAddress("glGetVertexAttribPointerv");
	  if(NULL == glGetVertexAttribPointerv) return false;

	  glIsProgram = (PFNGLISPROGRAMPROC)wglGetProcAddress("glIsProgram");
	  if(NULL == glIsProgram) return false;

	  glIsShader = (PFNGLISSHADERPROC)wglGetProcAddress("glIsShader");
	  if(NULL == glIsShader) return false;

	  glLinkProgram = (PFNGLLINKPROGRAMPROC)wglGetProcAddress("glLinkProgram");
	  if(NULL == glLinkProgram) return false;

	  glShaderSource = (PFNGLSHADERSOURCEPROC)wglGetProcAddress("glShaderSource");
	  if(NULL == glShaderSource) return false;

	  glUseProgram = (PFNGLUSEPROGRAMPROC)wglGetProcAddress("glUseProgram");
	  if(NULL == glUseProgram) return false;

	  glUniform1f = (PFNGLUNIFORM1FPROC)wglGetProcAddress("glUniform1f");
	  if(NULL == glUniform1f) return false;

	  glUniform2f = (PFNGLUNIFORM2FPROC)wglGetProcAddress("glUniform2f");
	  if(NULL == glUniform2f) return false;

	  glUniform3f = (PFNGLUNIFORM3FPROC)wglGetProcAddress("glUniform3f");
	  if(NULL == glUniform3f) return false;

	  glUniform4f = (PFNGLUNIFORM4FPROC)wglGetProcAddress("glUniform4f");
	  if(NULL == glUniform4f) return false;

	  glUniform1i = (PFNGLUNIFORM1IPROC)wglGetProcAddress("glUniform1i");
	  if(NULL == glUniform1i) return false;

	  glUniform2i = (PFNGLUNIFORM2IPROC)wglGetProcAddress("glUniform2i");
	  if(NULL == glUniform2i) return false;

	  glUniform3i = (PFNGLUNIFORM3IPROC)wglGetProcAddress("glUniform3i");
	  if(NULL == glUniform3i) return false;

	  glUniform4i = (PFNGLUNIFORM4IPROC)wglGetProcAddress("glUniform4i");
	  if(NULL == glUniform4i) return false;

	  glUniform1fv = (PFNGLUNIFORM1FVPROC)wglGetProcAddress("glUniform1fv");
	  if(NULL == glUniform1fv) return false;

	  glUniform2fv = (PFNGLUNIFORM2FVPROC)wglGetProcAddress("glUniform2fv");
	  if(NULL == glUniform2fv) return false;

	  glUniform3fv = (PFNGLUNIFORM3FVPROC)wglGetProcAddress("glUniform3fv");
	  if(NULL == glUniform3fv) return false;

	  glUniform4fv = (PFNGLUNIFORM4FVPROC)wglGetProcAddress("glUniform4fv");
	  if(NULL == glUniform4fv) return false;

	  glUniform1iv = (PFNGLUNIFORM1IVPROC)wglGetProcAddress("glUniform1iv");
	  if(NULL == glUniform1iv) return false;

	  glUniform2iv = (PFNGLUNIFORM2IVPROC)wglGetProcAddress("glUniform2iv");
	  if(NULL == glUniform2iv) return false;

	  glUniform3iv = (PFNGLUNIFORM3IVPROC)wglGetProcAddress("glUniform3iv");
	  if(NULL == glUniform3iv) return false;

	  glUniform4iv = (PFNGLUNIFORM4IVPROC)wglGetProcAddress("glUniform4iv");
	  if(NULL == glUniform4iv) return false;

	  glUniformMatrix2fv = (PFNGLUNIFORMMATRIX2FVPROC)wglGetProcAddress("glUniformMatrix2fv");
	  if(NULL == glUniformMatrix2fv) return false;

	  glUniformMatrix3fv = (PFNGLUNIFORMMATRIX3FVPROC)wglGetProcAddress("glUniformMatrix3fv");
	  if(NULL == glUniformMatrix3fv) return false;

	  glUniformMatrix4fv = (PFNGLUNIFORMMATRIX4FVPROC)wglGetProcAddress("glUniformMatrix4fv");
	  if(NULL == glUniformMatrix4fv) return false;

	  glValidateProgram = (PFNGLVALIDATEPROGRAMPROC)wglGetProcAddress("glValidateProgram");
	  if(NULL == glValidateProgram) return false;

	  glVertexAttrib1d = (PFNGLVERTEXATTRIB1DPROC)wglGetProcAddress("glVertexAttrib1d");
	  if(NULL == glVertexAttrib1d) return false;

	  glVertexAttrib1dv = (PFNGLVERTEXATTRIB1DVPROC)wglGetProcAddress("glVertexAttrib1dv");
	  if(NULL == glVertexAttrib1dv) return false;

	  glVertexAttrib1f = (PFNGLVERTEXATTRIB1FPROC)wglGetProcAddress("glVertexAttrib1f");
	  if(NULL == glVertexAttrib1f) return false;

	  glVertexAttrib1fv = (PFNGLVERTEXATTRIB1FVPROC)wglGetProcAddress("glVertexAttrib1fv");
	  if(NULL == glVertexAttrib1fv) return false;

	  glVertexAttrib1s = (PFNGLVERTEXATTRIB1SPROC)wglGetProcAddress("glVertexAttrib1s");
	  if(NULL == glVertexAttrib1s) return false;

	  glVertexAttrib1sv = (PFNGLVERTEXATTRIB1SVPROC)wglGetProcAddress("glVertexAttrib1sv");
	  if(NULL == glVertexAttrib1sv) return false;

	  glVertexAttrib2d = (PFNGLVERTEXATTRIB2DPROC)wglGetProcAddress("glVertexAttrib2d");
	  if(NULL == glVertexAttrib2d) return false;

	  glVertexAttrib2dv = (PFNGLVERTEXATTRIB2DVPROC)wglGetProcAddress("glVertexAttrib2dv");
	  if(NULL == glVertexAttrib2dv) return false;

	  glVertexAttrib2f = (PFNGLVERTEXATTRIB2FPROC)wglGetProcAddress("glVertexAttrib2f");
	  if(NULL == glVertexAttrib2f) return false;

	  glVertexAttrib2fv = (PFNGLVERTEXATTRIB2FVPROC)wglGetProcAddress("glVertexAttrib2fv");
	  if(NULL == glVertexAttrib2fv) return false;

	  glVertexAttrib2s = (PFNGLVERTEXATTRIB2SPROC)wglGetProcAddress("glVertexAttrib2s");
	  if(NULL == glVertexAttrib2s) return false;

	  glVertexAttrib2sv = (PFNGLVERTEXATTRIB2SVPROC)wglGetProcAddress("glVertexAttrib2sv");
	  if(NULL == glVertexAttrib2sv) return false;

	  glVertexAttrib3d = (PFNGLVERTEXATTRIB3DPROC)wglGetProcAddress("glVertexAttrib3d");
	  if(NULL == glVertexAttrib3d) return false;

	  glVertexAttrib3dv = (PFNGLVERTEXATTRIB3DVPROC)wglGetProcAddress("glVertexAttrib3dv");
	  if(NULL == glVertexAttrib3dv) return false;

	  glVertexAttrib3f = (PFNGLVERTEXATTRIB3FPROC)wglGetProcAddress("glVertexAttrib3f");
	  if(NULL == glVertexAttrib3f) return false;

	  glVertexAttrib3fv = (PFNGLVERTEXATTRIB3FVPROC)wglGetProcAddress("glVertexAttrib3fv");
	  if(NULL == glVertexAttrib3fv) return false;

	  glVertexAttrib3s = (PFNGLVERTEXATTRIB3SPROC)wglGetProcAddress("glVertexAttrib3s");
	  if(NULL == glVertexAttrib3s) return false;

	  glVertexAttrib3sv = (PFNGLVERTEXATTRIB3SVPROC)wglGetProcAddress("glVertexAttrib3sv");
	  if(NULL == glVertexAttrib3sv) return false;

	  glVertexAttrib4Nbv = (PFNGLVERTEXATTRIB4NBVPROC)wglGetProcAddress("glVertexAttrib4Nbv");
	  if(NULL == glVertexAttrib4Nbv) return false;

	  glVertexAttrib4Niv = (PFNGLVERTEXATTRIB4NIVPROC)wglGetProcAddress("glVertexAttrib4Niv");
	  if(NULL == glVertexAttrib4Niv) return false;

	  glVertexAttrib4Nsv = (PFNGLVERTEXATTRIB4NSVPROC)wglGetProcAddress("glVertexAttrib4Nsv");
	  if(NULL == glVertexAttrib4Nsv) return false;

	  glVertexAttrib4Nub = (PFNGLVERTEXATTRIB4NUBPROC)wglGetProcAddress("glVertexAttrib4Nub");
	  if(NULL == glVertexAttrib4Nub) return false;

	  glVertexAttrib4Nubv = (PFNGLVERTEXATTRIB4NUBVPROC)wglGetProcAddress("glVertexAttrib4Nubv");
	  if(NULL == glVertexAttrib4Nubv) return false;

	  glVertexAttrib4Nuiv = (PFNGLVERTEXATTRIB4NUIVPROC)wglGetProcAddress("glVertexAttrib4Nuiv");
	  if(NULL == glVertexAttrib4Nuiv) return false;

	  glVertexAttrib4Nusv = (PFNGLVERTEXATTRIB4NUSVPROC)wglGetProcAddress("glVertexAttrib4Nusv");
	  if(NULL == glVertexAttrib4Nusv) return false;

	  glVertexAttrib4bv = (PFNGLVERTEXATTRIB4BVPROC)wglGetProcAddress("glVertexAttrib4bv");
	  if(NULL == glVertexAttrib4bv) return false;

	  glVertexAttrib4d = (PFNGLVERTEXATTRIB4DPROC)wglGetProcAddress("glVertexAttrib4d");
	  if(NULL == glVertexAttrib4d) return false;

	  glVertexAttrib4dv = (PFNGLVERTEXATTRIB4DVPROC)wglGetProcAddress("glVertexAttrib4dv");
	  if(NULL == glVertexAttrib4dv) return false;

	  glVertexAttrib4f = (PFNGLVERTEXATTRIB4FPROC)wglGetProcAddress("glVertexAttrib4f");
	  if(NULL == glVertexAttrib4f) return false;

	  glVertexAttrib4fv = (PFNGLVERTEXATTRIB4FVPROC)wglGetProcAddress("glVertexAttrib4fv");
	  if(NULL == glVertexAttrib4fv) return false;

	  glVertexAttrib4iv = (PFNGLVERTEXATTRIB4IVPROC)wglGetProcAddress("glVertexAttrib4iv");
	  if(NULL == glVertexAttrib4iv) return false;

	  glVertexAttrib4s = (PFNGLVERTEXATTRIB4SPROC)wglGetProcAddress("glVertexAttrib4s");
	  if(NULL == glVertexAttrib4s) return false;

	  glVertexAttrib4sv = (PFNGLVERTEXATTRIB4SVPROC)wglGetProcAddress("glVertexAttrib4sv");
	  if(NULL == glVertexAttrib4sv) return false;

	  glVertexAttrib4ubv = (PFNGLVERTEXATTRIB4UBVPROC)wglGetProcAddress("glVertexAttrib4ubv");
	  if(NULL == glVertexAttrib4ubv) return false;

	  glVertexAttrib4uiv = (PFNGLVERTEXATTRIB4UIVPROC)wglGetProcAddress("glVertexAttrib4uiv");
	  if(NULL == glVertexAttrib4uiv) return false;

	  glVertexAttrib4usv = (PFNGLVERTEXATTRIB4USVPROC)wglGetProcAddress("glVertexAttrib4usv");
	  if(NULL == glVertexAttrib4usv) return false;

    glVertexAttribPointer = (PFNGLVERTEXATTRIBPOINTERPROC)wglGetProcAddress("glVertexAttribPointer");
	  if(NULL == glVertexAttribPointer) return false;
  }
  #endif



  #ifdef GL_ARB_vertex_shader
  else if(0 == strcmp(extension, "GL_ARB_vertex_shader"))
  {
	  glBindAttribLocationARB = (PFNGLBINDATTRIBLOCATIONARBPROC)wglGetProcAddress("glBindAttribLocationARB");
	  if(NULL == glBindAttribLocationARB) return false;

	  glGetActiveAttribARB = (PFNGLGETACTIVEATTRIBARBPROC)wglGetProcAddress("glGetActiveAttribARB");
	  if(NULL == glGetActiveAttribARB) return false;

	  glGetAttribLocationARB = (PFNGLGETATTRIBLOCATIONARBPROC)wglGetProcAddress("glGetAttribLocationARB");
	  if(NULL == glGetAttribLocationARB) return false;
  }
  #endif

  #ifdef GL_ARB_shader_objects
  else if(0 == strcmp(extension, "GL_ARB_shader_objects"))
  {
	  glDeleteObjectARB = (PFNGLDELETEOBJECTARBPROC)wglGetProcAddress("glDeleteObjectARB");
	  if(NULL == glDeleteObjectARB) return false;

	  glGetHandleARB = (PFNGLGETHANDLEARBPROC)wglGetProcAddress("glGetHandleARB");
	  if(NULL == glGetHandleARB) return false;

	  glDetachObjectARB = (PFNGLDETACHOBJECTARBPROC)wglGetProcAddress("glDetachObjectARB");
	  if(NULL == glDetachObjectARB) return false;

    glCreateShaderObjectARB = (PFNGLCREATESHADEROBJECTARBPROC)wglGetProcAddress("glCreateShaderObjectARB");
	  if(NULL == glCreateShaderObjectARB) return false;

	  glShaderSourceARB = (PFNGLSHADERSOURCEARBPROC)wglGetProcAddress("glShaderSourceARB");
	  if(NULL == glShaderSourceARB) return false;

	  glCompileShaderARB = (PFNGLCOMPILESHADERARBPROC)wglGetProcAddress("glCompileShaderARB");
	  if(NULL == glCompileShaderARB) return false;

    glCreateProgramObjectARB = (PFNGLCREATEPROGRAMOBJECTARBPROC)wglGetProcAddress("glCreateProgramObjectARB");
	  if(NULL == glCreateProgramObjectARB) return false;

	  glAttachObjectARB = (PFNGLATTACHOBJECTARBPROC)wglGetProcAddress("glAttachObjectARB");
	  if(NULL == glAttachObjectARB) return false;

	  glLinkProgramARB = (PFNGLLINKPROGRAMARBPROC)wglGetProcAddress("glLinkProgramARB");
	  if(NULL == glLinkProgramARB) return false;

	  glUseProgramObjectARB = (PFNGLUSEPROGRAMOBJECTARBPROC)wglGetProcAddress("glUseProgramObjectARB");
	  if(NULL == glUseProgramObjectARB) return false;

	  glValidateProgramARB = (PFNGLVALIDATEPROGRAMARBPROC)wglGetProcAddress("glValidateProgramARB");
	  if(NULL == glValidateProgramARB) return false;

	  glUniform1fARB = (PFNGLUNIFORM1FARBPROC)wglGetProcAddress("glUniform1fARB");
	  if(NULL == glUniform1fARB) return false;

	  glUniform2fARB = (PFNGLUNIFORM2FARBPROC)wglGetProcAddress("glUniform2fARB");
	  if(NULL == glUniform2fARB) return false;

	  glUniform3fARB = (PFNGLUNIFORM3FARBPROC)wglGetProcAddress("glUniform3fARB");
	  if(NULL == glUniform3fARB) return false;

	  glUniform4fARB = (PFNGLUNIFORM4FARBPROC)wglGetProcAddress("glUniform4fARB");
	  if(NULL == glUniform4fARB) return false;

	  glUniform1iARB = (PFNGLUNIFORM1IARBPROC)wglGetProcAddress("glUniform1iARB");
	  if(NULL == glUniform1iARB) return false;

	  glUniform2iARB = (PFNGLUNIFORM2IARBPROC)wglGetProcAddress("glUniform2iARB");
	  if(NULL == glUniform2iARB) return false;

	  glUniform3iARB = (PFNGLUNIFORM3IARBPROC)wglGetProcAddress("glUniform3iARB");
	  if(NULL == glUniform3iARB) return false;

	  glUniform4iARB = (PFNGLUNIFORM4IARBPROC)wglGetProcAddress("glUniform4iARB");
	  if(NULL == glUniform4iARB) return false;

	  glUniform1fvARB = (PFNGLUNIFORM1FVARBPROC)wglGetProcAddress("glUniform1fvARB");
	  if(NULL == glUniform1fvARB) return false;

	  glUniform2fvARB = (PFNGLUNIFORM2FVARBPROC)wglGetProcAddress("glUniform2fvARB");
	  if(NULL == glUniform2fvARB) return false;

	  glUniform3fvARB = (PFNGLUNIFORM3FVARBPROC)wglGetProcAddress("glUniform3fvARB");
	  if(NULL == glUniform3fvARB) return false;

	  glUniform4fvARB = (PFNGLUNIFORM4FVARBPROC)wglGetProcAddress("glUniform4fvARB");
	  if(NULL == glUniform4fvARB) return false;

    glUniform1ivARB = (PFNGLUNIFORM1IVARBPROC)wglGetProcAddress("glUniform1ivARB");
	  if(NULL == glUniform1ivARB) return false;

	  glUniform2ivARB = (PFNGLUNIFORM2IVARBPROC)wglGetProcAddress("glUniform2ivARB");
	  if(NULL == glUniform2ivARB) return false;

	  glUniform3ivARB = (PFNGLUNIFORM3IVARBPROC)wglGetProcAddress("glUniform3ivARB");
	  if(NULL == glUniform3ivARB) return false;

	  glUniform4ivARB = (PFNGLUNIFORM4IVARBPROC)wglGetProcAddress("glUniform4ivARB");
	  if(NULL == glUniform4ivARB) return false;

	  glUniformMatrix2fvARB = (PFNGLUNIFORMMATRIX2FVARBPROC)wglGetProcAddress("glUniformMatrix2fvARB");
	  if(NULL == glUniformMatrix2fvARB) return false;

	  glUniformMatrix3fvARB = (PFNGLUNIFORMMATRIX3FVARBPROC)wglGetProcAddress("glUniformMatrix3fvARB");
	  if(NULL == glUniformMatrix3fvARB) return false;

	  glUniformMatrix4fvARB = (PFNGLUNIFORMMATRIX4FVARBPROC)wglGetProcAddress("glUniformMatrix4fvARB");
	  if(NULL == glUniformMatrix4fvARB) return false;

	  glGetObjectParameterfvARB = (PFNGLGETOBJECTPARAMETERFVARBPROC)wglGetProcAddress("glGetObjectParameterfvARB");
	  if(NULL == glGetObjectParameterfvARB) return false;

	  glGetObjectParameterivARB = (PFNGLGETOBJECTPARAMETERIVARBPROC)wglGetProcAddress("glGetObjectParameterivARB");
	  if(NULL == glGetObjectParameterivARB) return false;

    glGetInfoLogARB = (PFNGLGETINFOLOGARBPROC)wglGetProcAddress("glGetInfoLogARB");
	  if(NULL == glGetInfoLogARB) return false;

	  glGetAttachedObjectsARB = (PFNGLGETATTACHEDOBJECTSARBPROC)wglGetProcAddress("glGetAttachedObjectsARB");
	  if(NULL == glGetAttachedObjectsARB) return false;

	  glGetUniformLocationARB = (PFNGLGETUNIFORMLOCATIONARBPROC)wglGetProcAddress("glGetUniformLocationARB");
	  if(NULL == glGetUniformLocationARB) return false;

	  glGetActiveUniformARB = (PFNGLGETACTIVEUNIFORMARBPROC)wglGetProcAddress("glGetActiveUniformARB");
	  if(NULL == glGetActiveUniformARB) return false;

    glGetUniformfvARB = (PFNGLGETUNIFORMFVARBPROC)wglGetProcAddress("glGetUniformfvARB");
	  if(NULL == glGetUniformfvARB) return false;

	  glGetUniformivARB = (PFNGLGETUNIFORMIVARBPROC)wglGetProcAddress("glGetUniformivARB");
	  if(NULL == glGetUniformivARB) return false;

	  glGetShaderSourceARB = (PFNGLGETSHADERSOURCEARBPROC)wglGetProcAddress("glGetShaderSourceARB");
	  if(NULL == glGetShaderSourceARB) return false;
  }
  #endif

  #ifdef GL_ARB_vertex_program
  else if(0 == strcmp(extension, "GL_ARB_vertex_program"))
  {
	  glVertexAttrib1dARB = (PFNGLVERTEXATTRIB1DARBPROC)wglGetProcAddress("glVertexAttrib1dARB");
	  if(NULL == glVertexAttrib1dARB) return false;

	  glVertexAttrib1dvARB = (PFNGLVERTEXATTRIB1DVARBPROC)wglGetProcAddress("glVertexAttrib1dvARB");
	  if(NULL == glVertexAttrib1dvARB) return false;

	  glVertexAttrib1fARB = (PFNGLVERTEXATTRIB1FARBPROC)wglGetProcAddress("glVertexAttrib1fARB");
	  if(NULL == glVertexAttrib1fARB) return false;

    glVertexAttrib1fvARB = (PFNGLVERTEXATTRIB1FVARBPROC)wglGetProcAddress("glVertexAttrib1fvARB");
	  if(NULL == glVertexAttrib1fvARB) return false;

	  glVertexAttrib1sARB = (PFNGLVERTEXATTRIB1SARBPROC)wglGetProcAddress("glVertexAttrib1sARB");
	  if(NULL == glVertexAttrib1sARB) return false;

	  glVertexAttrib1svARB = (PFNGLVERTEXATTRIB1SVARBPROC)wglGetProcAddress("glVertexAttrib1svARB");
	  if(NULL == glVertexAttrib1svARB) return false;

    glVertexAttrib2dARB = (PFNGLVERTEXATTRIB2DARBPROC)wglGetProcAddress("glVertexAttrib2dARB");
	  if(NULL == glVertexAttrib2dARB) return false;

	  glVertexAttrib2dvARB = (PFNGLVERTEXATTRIB2DVARBPROC)wglGetProcAddress("glVertexAttrib2dvARB");
	  if(NULL == glVertexAttrib2dvARB) return false;

	  glVertexAttrib2fARB = (PFNGLVERTEXATTRIB2FARBPROC)wglGetProcAddress("glVertexAttrib2fARB");
	  if(NULL == glVertexAttrib2fARB) return false;

    glVertexAttrib2fvARB = (PFNGLVERTEXATTRIB2FVARBPROC)wglGetProcAddress("glVertexAttrib2fvARB");
	  if(NULL == glVertexAttrib2fvARB) return false;

	  glVertexAttrib2sARB = (PFNGLVERTEXATTRIB2SARBPROC)wglGetProcAddress("glVertexAttrib2sARB");
	  if(NULL == glVertexAttrib2sARB) return false;

	  glVertexAttrib2svARB = (PFNGLVERTEXATTRIB2SVARBPROC)wglGetProcAddress("glVertexAttrib2svARB");
	  if(NULL == glVertexAttrib2svARB) return false;

    glVertexAttrib3dARB = (PFNGLVERTEXATTRIB3DARBPROC)wglGetProcAddress("glVertexAttrib3dARB");
	  if(NULL == glVertexAttrib3dARB) return false;

	  glVertexAttrib3dvARB = (PFNGLVERTEXATTRIB3DVARBPROC)wglGetProcAddress("glVertexAttrib3dvARB");
	  if(NULL == glVertexAttrib3dvARB) return false;

	  glVertexAttrib3fARB = (PFNGLVERTEXATTRIB3FARBPROC)wglGetProcAddress("glVertexAttrib3fARB");
	  if(NULL == glVertexAttrib3fARB) return false;

    glVertexAttrib3fvARB = (PFNGLVERTEXATTRIB3FVARBPROC)wglGetProcAddress("glVertexAttrib3fvARB");
	  if(NULL == glVertexAttrib3fvARB) return false;

	  glVertexAttrib3sARB = (PFNGLVERTEXATTRIB3SARBPROC)wglGetProcAddress("glVertexAttrib3sARB");
	  if(NULL == glVertexAttrib3sARB) return false;

	  glVertexAttrib3svARB = (PFNGLVERTEXATTRIB3SVARBPROC)wglGetProcAddress("glVertexAttrib3svARB");
	  if(NULL == glVertexAttrib3svARB) return false;

    glVertexAttrib4NbvARB = (PFNGLVERTEXATTRIB4NBVARBPROC)wglGetProcAddress("glVertexAttrib4NbvARB");
	  if(NULL == glVertexAttrib4NbvARB) return false;

	  glVertexAttrib4NivARB = (PFNGLVERTEXATTRIB4NIVARBPROC)wglGetProcAddress("glVertexAttrib4NivARB");
	  if(NULL == glVertexAttrib4NivARB) return false;

	  glVertexAttrib4NsvARB = (PFNGLVERTEXATTRIB4NSVARBPROC)wglGetProcAddress("glVertexAttrib4NsvARB");
	  if(NULL == glVertexAttrib4NsvARB) return false;

    glVertexAttrib4NubARB = (PFNGLVERTEXATTRIB4NUBARBPROC)wglGetProcAddress("glVertexAttrib4NubARB");
	  if(NULL == glVertexAttrib4NubARB) return false;

	  glVertexAttrib4NubvARB = (PFNGLVERTEXATTRIB4NUBVARBPROC)wglGetProcAddress("glVertexAttrib4NubvARB");
	  if(NULL == glVertexAttrib4NubvARB) return false;

	  glVertexAttrib4NuivARB = (PFNGLVERTEXATTRIB4NUIVARBPROC)wglGetProcAddress("glVertexAttrib4NuivARB");
	  if(NULL == glVertexAttrib4NuivARB) return false;

    glVertexAttrib4NusvARB = (PFNGLVERTEXATTRIB4NUSVARBPROC)wglGetProcAddress("glVertexAttrib4NusvARB");
	  if(NULL == glVertexAttrib4NusvARB) return false;

	  glVertexAttrib4bvARB = (PFNGLVERTEXATTRIB4BVARBPROC)wglGetProcAddress("glVertexAttrib4bvARB");
	  if(NULL == glVertexAttrib4bvARB) return false;

	  glVertexAttrib4dARB = (PFNGLVERTEXATTRIB4DARBPROC)wglGetProcAddress("glVertexAttrib4dARB");
	  if(NULL == glVertexAttrib4dARB) return false;

    glVertexAttrib4dvARB = (PFNGLVERTEXATTRIB4DVARBPROC)wglGetProcAddress("glVertexAttrib4dvARB");
	  if(NULL == glVertexAttrib4dvARB) return false;

	  glVertexAttrib4fARB = (PFNGLVERTEXATTRIB4FARBPROC)wglGetProcAddress("glVertexAttrib4fARB");
	  if(NULL == glVertexAttrib4fARB) return false;

	  glVertexAttrib4fvARB = (PFNGLVERTEXATTRIB4FVARBPROC)wglGetProcAddress("glVertexAttrib4fvARB");
	  if(NULL == glVertexAttrib4fvARB) return false;

    glVertexAttrib4ivARB = (PFNGLVERTEXATTRIB4IVARBPROC)wglGetProcAddress("glVertexAttrib4ivARB");
	  if(NULL == glVertexAttrib4ivARB) return false;

	  glVertexAttrib4sARB = (PFNGLVERTEXATTRIB4SARBPROC)wglGetProcAddress("glVertexAttrib4sARB");
	  if(NULL == glVertexAttrib4sARB) return false;

	  glVertexAttrib4svARB = (PFNGLVERTEXATTRIB4SVARBPROC)wglGetProcAddress("glVertexAttrib4svARB");
	  if(NULL == glVertexAttrib4svARB) return false;

    glVertexAttrib4ubvARB = (PFNGLVERTEXATTRIB4UBVARBPROC)wglGetProcAddress("glVertexAttrib4ubvARB");
	  if(NULL == glVertexAttrib4ubvARB) return false;

	  glVertexAttrib4uivARB = (PFNGLVERTEXATTRIB4UIVARBPROC)wglGetProcAddress("glVertexAttrib4uivARB");
	  if(NULL == glVertexAttrib4uivARB) return false;

	  glVertexAttrib4usvARB = (PFNGLVERTEXATTRIB4USVARBPROC)wglGetProcAddress("glVertexAttrib4usvARB");
	  if(NULL == glVertexAttrib4usvARB) return false;

    glVertexAttribPointerARB = (PFNGLVERTEXATTRIBPOINTERARBPROC)wglGetProcAddress("glVertexAttribPointerARB");
	  if(NULL == glVertexAttribPointerARB) return false;

	  glEnableVertexAttribArrayARB = (PFNGLENABLEVERTEXATTRIBARRAYARBPROC)wglGetProcAddress("glEnableVertexAttribArrayARB");
	  if(NULL == glEnableVertexAttribArrayARB) return false;

	  glDisableVertexAttribArrayARB = (PFNGLDISABLEVERTEXATTRIBARRAYARBPROC)wglGetProcAddress("glDisableVertexAttribArrayARB");
	  if(NULL == glDisableVertexAttribArrayARB) return false;

    glProgramStringARB = (PFNGLPROGRAMSTRINGARBPROC)wglGetProcAddress("glProgramStringARB");
	  if(NULL == glProgramStringARB) return false;

	  glBindProgramARB = (PFNGLBINDPROGRAMARBPROC)wglGetProcAddress("glBindProgramARB");
	  if(NULL == glBindProgramARB) return false;

	  glDeleteProgramsARB = (PFNGLDELETEPROGRAMSARBPROC)wglGetProcAddress("glDeleteProgramsARB");
	  if(NULL == glDeleteProgramsARB) return false;

    glGenProgramsARB = (PFNGLGENPROGRAMSARBPROC)wglGetProcAddress("glGenProgramsARB");
	  if(NULL == glGenProgramsARB) return false;

	  glProgramEnvParameter4dARB = (PFNGLPROGRAMENVPARAMETER4DARBPROC)wglGetProcAddress("glProgramEnvParameter4dARB");
	  if(NULL == glProgramEnvParameter4dARB) return false;

	  glProgramEnvParameter4dvARB = (PFNGLPROGRAMENVPARAMETER4DVARBPROC)wglGetProcAddress("glProgramEnvParameter4dvARB");
	  if(NULL == glProgramEnvParameter4dvARB) return false;

    glProgramEnvParameter4fARB = (PFNGLPROGRAMENVPARAMETER4FARBPROC)wglGetProcAddress("glProgramEnvParameter4fARB");
	  if(NULL == glProgramEnvParameter4fARB) return false;

	  glProgramEnvParameter4fvARB = (PFNGLPROGRAMENVPARAMETER4FVARBPROC)wglGetProcAddress("glProgramEnvParameter4fvARB");
	  if(NULL == glProgramEnvParameter4fvARB) return false;

	  glProgramLocalParameter4dARB = (PFNGLPROGRAMLOCALPARAMETER4DARBPROC)wglGetProcAddress("glProgramLocalParameter4dARB");
	  if(NULL == glProgramLocalParameter4dARB) return false;

    glProgramLocalParameter4dvARB = (PFNGLPROGRAMLOCALPARAMETER4DVARBPROC)wglGetProcAddress("glProgramLocalParameter4dvARB");
	  if(NULL == glProgramLocalParameter4dvARB) return false;

	  glProgramLocalParameter4fARB = (PFNGLPROGRAMLOCALPARAMETER4FARBPROC)wglGetProcAddress("glProgramLocalParameter4fARB");
	  if(NULL == glProgramLocalParameter4fARB) return false;

	  glProgramLocalParameter4fvARB = (PFNGLPROGRAMLOCALPARAMETER4FVARBPROC)wglGetProcAddress("glProgramLocalParameter4fvARB");
	  if(NULL == glProgramLocalParameter4fvARB) return false;

    glGetProgramEnvParameterdvARB = (PFNGLGETPROGRAMENVPARAMETERDVARBPROC)wglGetProcAddress("glGetProgramEnvParameterdvARB");
	  if(NULL == glGetProgramEnvParameterdvARB) return false;

	  glGetProgramEnvParameterfvARB = (PFNGLGETPROGRAMENVPARAMETERFVARBPROC)wglGetProcAddress("glGetProgramEnvParameterfvARB");
	  if(NULL == glGetProgramEnvParameterfvARB) return false;

	  glGetProgramLocalParameterdvARB = (PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC)wglGetProcAddress("glGetProgramLocalParameterdvARB");
	  if(NULL == glGetProgramLocalParameterdvARB) return false;

    glGetProgramLocalParameterfvARB = (PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC)wglGetProcAddress("glGetProgramLocalParameterfvARB");
	  if(NULL == glGetProgramLocalParameterfvARB) return false;

	  glGetProgramivARB = (PFNGLGETPROGRAMIVARBPROC)wglGetProcAddress("glGetProgramivARB");
	  if(NULL == glGetProgramivARB) return false;

	  glGetProgramStringARB = (PFNGLGETPROGRAMSTRINGARBPROC)wglGetProcAddress("glGetProgramStringARB");
	  if(NULL == glGetProgramStringARB) return false;

    glGetVertexAttribdvARB = (PFNGLGETVERTEXATTRIBDVARBPROC)wglGetProcAddress("glGetVertexAttribdvARB");
	  if(NULL == glGetVertexAttribdvARB) return false;

	  glGetVertexAttribfvARB = (PFNGLGETVERTEXATTRIBFVARBPROC)wglGetProcAddress("glGetVertexAttribfvARB");
	  if(NULL == glGetVertexAttribfvARB) return false;

	  glGetVertexAttribivARB = (PFNGLGETVERTEXATTRIBIVARBPROC)wglGetProcAddress("glGetVertexAttribivARB");
	  if(NULL == glGetVertexAttribivARB) return false;

	  glGetVertexAttribPointervARB = (PFNGLGETVERTEXATTRIBPOINTERVARBPROC)wglGetProcAddress("glGetVertexAttribPointervARB");
	  if(NULL == glGetVertexAttribPointervARB) return false;

	  glIsProgramARB = (PFNGLISPROGRAMARBPROC)wglGetProcAddress("glIsProgramARB");
	  if(NULL == glIsProgramARB) return false;
  }
  #endif

  #ifdef GL_ARB_multitexture
  else if(0 == strcmp(extension, "GL_ARB_multitexture"))
  {
	  glActiveTextureARB = (PFNGLACTIVETEXTUREARBPROC)wglGetProcAddress("glActiveTextureARB");
	  if(NULL == glActiveTextureARB) return false;

	  glClientActiveTextureARB = (PFNGLCLIENTACTIVETEXTUREARBPROC)wglGetProcAddress("glClientActiveTextureARB");
	  if(NULL == glClientActiveTextureARB) return false;

	  glMultiTexCoord1dARB = (PFNGLMULTITEXCOORD1DARBPROC)wglGetProcAddress("glMultiTexCoord1dARB");
	  if(NULL == glMultiTexCoord1dARB) return false;

	  glMultiTexCoord1dvARB = (PFNGLMULTITEXCOORD1DVARBPROC)wglGetProcAddress("glMultiTexCoord1dvARB");
	  if(NULL == glMultiTexCoord1dvARB) return false;

	  glMultiTexCoord1fARB = (PFNGLMULTITEXCOORD1FARBPROC)wglGetProcAddress("glMultiTexCoord1fARB");
	  if(NULL == glMultiTexCoord1fARB) return false;

	  glMultiTexCoord1fvARB = (PFNGLMULTITEXCOORD1FVARBPROC)wglGetProcAddress("glMultiTexCoord1fvARB");
	  if(NULL == glMultiTexCoord1fvARB) return false;

	  glMultiTexCoord1iARB = (PFNGLMULTITEXCOORD1IARBPROC)wglGetProcAddress("glMultiTexCoord1iARB");
	  if(NULL == glMultiTexCoord1iARB) return false;

	  glMultiTexCoord1ivARB = (PFNGLMULTITEXCOORD1IVARBPROC)wglGetProcAddress("glMultiTexCoord1ivARB");
	  if(NULL == glMultiTexCoord1ivARB) return false;

	  glMultiTexCoord1sARB = (PFNGLMULTITEXCOORD1SARBPROC)wglGetProcAddress("glMultiTexCoord1sARB");
	  if(NULL == glMultiTexCoord1sARB) return false;

	  glMultiTexCoord1svARB = (PFNGLMULTITEXCOORD1SVARBPROC)wglGetProcAddress("glMultiTexCoord1svARB");
	  if(NULL == glMultiTexCoord1svARB) return false;

	  glMultiTexCoord2dARB = (PFNGLMULTITEXCOORD2DARBPROC)wglGetProcAddress("glMultiTexCoord2dARB");
	  if(NULL == glMultiTexCoord2dARB) return false;

	  glMultiTexCoord2dvARB = (PFNGLMULTITEXCOORD2DVARBPROC)wglGetProcAddress("glMultiTexCoord2dvARB");
	  if(NULL == glMultiTexCoord2dvARB) return false;

	  glMultiTexCoord2fARB = (PFNGLMULTITEXCOORD2FARBPROC)wglGetProcAddress("glMultiTexCoord2fARB");
	  if(NULL == glMultiTexCoord2fARB) return false;

	  glMultiTexCoord2fvARB = (PFNGLMULTITEXCOORD2FVARBPROC)wglGetProcAddress("glMultiTexCoord2fvARB");
	  if(NULL == glMultiTexCoord2fvARB) return false;

	  glMultiTexCoord2iARB = (PFNGLMULTITEXCOORD2IARBPROC)wglGetProcAddress("glMultiTexCoord2iARB");
	  if(NULL == glMultiTexCoord2iARB) return false;

	  glMultiTexCoord2ivARB = (PFNGLMULTITEXCOORD2IVARBPROC)wglGetProcAddress("glMultiTexCoord2ivARB");
	  if(NULL == glMultiTexCoord2ivARB) return false;

	  glMultiTexCoord2sARB = (PFNGLMULTITEXCOORD2SARBPROC)wglGetProcAddress("glMultiTexCoord2sARB");
	  if(NULL == glMultiTexCoord2sARB) return false;

	  glMultiTexCoord2svARB = (PFNGLMULTITEXCOORD2SVARBPROC)wglGetProcAddress("glMultiTexCoord2svARB");
	  if(NULL == glMultiTexCoord2svARB) return false;

	  glMultiTexCoord3dARB = (PFNGLMULTITEXCOORD3DARBPROC)wglGetProcAddress("glMultiTexCoord3dARB");
	  if(NULL == glMultiTexCoord3dARB) return false;

	  glMultiTexCoord3dvARB = (PFNGLMULTITEXCOORD3DVARBPROC)wglGetProcAddress("glMultiTexCoord3dvARB");
	  if(NULL == glMultiTexCoord3dvARB) return false;

	  glMultiTexCoord3fARB = (PFNGLMULTITEXCOORD3FARBPROC)wglGetProcAddress("glMultiTexCoord3fARB");
	  if(NULL == glMultiTexCoord3fARB) return false;

	  glMultiTexCoord3fvARB = (PFNGLMULTITEXCOORD3FVARBPROC)wglGetProcAddress("glMultiTexCoord3fvARB");
	  if(NULL == glMultiTexCoord3fvARB) return false;

	  glMultiTexCoord3iARB = (PFNGLMULTITEXCOORD3IARBPROC)wglGetProcAddress("glMultiTexCoord3iARB");
	  if(NULL == glMultiTexCoord3iARB) return false;

	  glMultiTexCoord3ivARB = (PFNGLMULTITEXCOORD3IVARBPROC)wglGetProcAddress("glMultiTexCoord3ivARB");
	  if(NULL == glMultiTexCoord3ivARB) return false;

	  glMultiTexCoord3sARB = (PFNGLMULTITEXCOORD3SARBPROC)wglGetProcAddress("glMultiTexCoord3sARB");
	  if(NULL == glMultiTexCoord3sARB) return false;

	  glMultiTexCoord3svARB = (PFNGLMULTITEXCOORD3SVARBPROC)wglGetProcAddress("glMultiTexCoord3svARB");
	  if(NULL == glMultiTexCoord3svARB) return false;

	  glMultiTexCoord4dARB = (PFNGLMULTITEXCOORD4DARBPROC)wglGetProcAddress("glMultiTexCoord4dARB");
	  if(NULL == glMultiTexCoord4dARB) return false;

	  glMultiTexCoord4dvARB = (PFNGLMULTITEXCOORD4DVARBPROC)wglGetProcAddress("glMultiTexCoord4dvARB");
	  if(NULL == glMultiTexCoord4dvARB) return false;

	  glMultiTexCoord4fARB = (PFNGLMULTITEXCOORD4FARBPROC)wglGetProcAddress("glMultiTexCoord4fARB");
	  if(NULL == glMultiTexCoord4fARB) return false;

	  glMultiTexCoord4fvARB = (PFNGLMULTITEXCOORD4FVARBPROC)wglGetProcAddress("glMultiTexCoord4fvARB");
	  if(NULL == glMultiTexCoord4fvARB) return false;

	  glMultiTexCoord4iARB = (PFNGLMULTITEXCOORD4IARBPROC)wglGetProcAddress("glMultiTexCoord4iARB");
	  if(NULL == glMultiTexCoord4iARB) return false;

	  glMultiTexCoord4ivARB = (PFNGLMULTITEXCOORD4IVARBPROC)wglGetProcAddress("glMultiTexCoord4ivARB");
	  if(NULL == glMultiTexCoord4ivARB) return false;

	  glMultiTexCoord4sARB = (PFNGLMULTITEXCOORD4SARBPROC)wglGetProcAddress("glMultiTexCoord4sARB");
	  if(NULL == glMultiTexCoord4sARB) return false;

	  glMultiTexCoord4svARB = (PFNGLMULTITEXCOORD4SVARBPROC)wglGetProcAddress("glMultiTexCoord4svARB");
	  if(NULL == glMultiTexCoord4svARB) return false;
  }
  #endif

  #ifdef WGL_ARB_pbuffer
  else if(0 == strcmp(extension, "WGL_ARB_pbuffer"))
  {
	  wglCreatePbufferARB = (PFNWGLCREATEPBUFFERARBPROC)wglGetProcAddress("wglCreatePbufferARB");
	  if(NULL == wglCreatePbufferARB) return false;

	  wglGetPbufferDCARB = (PFNWGLGETPBUFFERDCARBPROC)wglGetProcAddress("wglGetPbufferDCARB");
	  if(NULL == wglGetPbufferDCARB) return false;

	  wglReleasePbufferDCARB = (PFNWGLRELEASEPBUFFERDCARBPROC)wglGetProcAddress("wglReleasePbufferDCARB");
	  if(NULL == wglReleasePbufferDCARB) return false;

	  wglDestroyPbufferARB = (PFNWGLDESTROYPBUFFERARBPROC)wglGetProcAddress("wglDestroyPbufferARB");
	  if(NULL == wglDestroyPbufferARB) return false;

	  wglQueryPbufferARB = (PFNWGLQUERYPBUFFERARBPROC)wglGetProcAddress("wglQueryPbufferARB");
	  if(NULL == wglQueryPbufferARB) return false;

  }
  #endif

  #ifdef GL_EXT_framebuffer_object
  else if(0 == strcmp(extension, "GL_EXT_framebuffer_object"))
  {
	  glIsRenderbufferEXT = (PFNGLISRENDERBUFFEREXTPROC)wglGetProcAddress("glIsRenderbufferEXT");
	  if(NULL == glIsRenderbufferEXT) return false;

	  glBindRenderbufferEXT = (PFNGLBINDRENDERBUFFEREXTPROC)wglGetProcAddress("glBindRenderbufferEXT");
	  if(NULL == glBindRenderbufferEXT) return false;

	  glDeleteRenderbuffersEXT = (PFNGLDELETERENDERBUFFERSEXTPROC)wglGetProcAddress("glDeleteRenderbuffersEXT");
	  if(NULL == glDeleteRenderbuffersEXT) return false;

	  glGenRenderbuffersEXT = (PFNGLGENRENDERBUFFERSEXTPROC)wglGetProcAddress("glGenRenderbuffersEXT");
	  if(NULL == glGenRenderbuffersEXT) return false;

	  glRenderbufferStorageEXT = (PFNGLRENDERBUFFERSTORAGEEXTPROC)wglGetProcAddress("glRenderbufferStorageEXT");
	  if(NULL == glRenderbufferStorageEXT) return false;

	  glGetRenderbufferParameterivEXT = (PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC)wglGetProcAddress("glGetRenderbufferParameterivEXT");
	  if(NULL == glGetRenderbufferParameterivEXT) return false;

	  glIsFramebufferEXT = (PFNGLISFRAMEBUFFEREXTPROC)wglGetProcAddress("glIsFramebufferEXT");
	  if(NULL == glIsFramebufferEXT) return false;

    glBindFramebufferEXT = (PFNGLBINDFRAMEBUFFEREXTPROC)wglGetProcAddress("glBindFramebufferEXT");
	  if(NULL == glBindFramebufferEXT) return false;

	  glDeleteFramebuffersEXT = (PFNGLDELETEFRAMEBUFFERSEXTPROC)wglGetProcAddress("glDeleteFramebuffersEXT");
	  if(NULL == glDeleteFramebuffersEXT) return false;


	  glGenFramebuffersEXT = (PFNGLGENFRAMEBUFFERSEXTPROC)wglGetProcAddress("glGenFramebuffersEXT");
	  if(NULL == glGenFramebuffersEXT) return false;

	  glCheckFramebufferStatusEXT = (PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC)wglGetProcAddress("glCheckFramebufferStatusEXT");
	  if(NULL == glCheckFramebufferStatusEXT) return false;

	  glFramebufferTexture1DEXT = (PFNGLFRAMEBUFFERTEXTURE1DEXTPROC)wglGetProcAddress("glFramebufferTexture1DEXT");
	  if(NULL == glFramebufferTexture1DEXT) return false;

	  glFramebufferTexture2DEXT = (PFNGLFRAMEBUFFERTEXTURE2DEXTPROC)wglGetProcAddress("glFramebufferTexture2DEXT");
	  if(NULL == glFramebufferTexture2DEXT) return false;

	  glFramebufferTexture3DEXT = (PFNGLFRAMEBUFFERTEXTURE3DEXTPROC)wglGetProcAddress("glFramebufferTexture3DEXT");
	  if(NULL == glFramebufferTexture3DEXT) return false;

	  glFramebufferRenderbufferEXT = (PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC)wglGetProcAddress("glFramebufferRenderbufferEXT");
	  if(NULL == glFramebufferRenderbufferEXT) return false;

	  glGetFramebufferAttachmentParameterivEXT = (PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC)wglGetProcAddress("glGetFramebufferAttachmentParameterivEXT");
	  if(NULL == glGetFramebufferAttachmentParameterivEXT) return false;

	  glGenerateMipmapEXT = (PFNGLGENERATEMIPMAPEXTPROC)wglGetProcAddress("glGenerateMipmapEXT");
	  if(NULL == glGenerateMipmapEXT) return false;
  }
  #endif


  #ifdef GL_ATI_draw_buffers
  else if(0 == strcmp(extension, "GL_ATI_draw_buffers"))
  {
	  glDrawBuffersATI = (PFNGLDRAWBUFFERSATIPROC)wglGetProcAddress("glDrawBuffersATI");
	  if(NULL == glDrawBuffersATI) return false;
  }
  #endif

  #ifdef GL_EXT_texture3D
  else if(0 == strcmp(extension, "GL_EXT_texture3D"))
  {
	  glTexImage3DEXT = (PFNGLTEXIMAGE3DEXTPROC)wglGetProcAddress("glTexImage3DEXT");
	  if(NULL == glTexImage3DEXT) return false;

    glTexSubImage3DEXT = (PFNGLTEXSUBIMAGE3DEXTPROC)wglGetProcAddress("glTexSubImage3DEXT");
	  if(NULL == glTexSubImage3DEXT) return false;
  }
  #endif
  
  #ifdef GL_VERSION_1_2
  else if(0 == strcmp(extension, "GL_VERSION_1_2"))
  {
	  glTexImage3D = (PFNGLTEXIMAGE3DPROC)wglGetProcAddress("glTexImage3D");
	  if(NULL == glTexImage3D) return false;

    glTexSubImage3D = (PFNGLTEXSUBIMAGE3DPROC)wglGetProcAddress("glTexSubImage3D");
	  if(NULL == glTexSubImage3D) return false;

    glCopyTexSubImage3D = (PFNGLCOPYTEXSUBIMAGE3DPROC)wglGetProcAddress("glCopyTexSubImage3D");
	  if(NULL == glCopyTexSubImage3D) return false;
  }
  #endif


  else return false;

	return true;
} // end generateExtension


bool isShaderExtensionSupported(const char *extension)
{
	const GLubyte *extensions = NULL;
	const GLubyte *start;
	GLubyte *where, *terminator;

	where = (GLubyte *) strchr(extension, ' ');
	
	if(where || *extension == '\0') return false;
	extensions = glGetString(GL_EXTENSIONS);
  
//  bool b = false;
//  if(b) printf("%s\n",extensions);
  start = extensions;
  
	for(;;) 
	{
		where = (GLubyte *) strstr((const char *) start, extension);
		if(!where) break;
    
		terminator = where + strlen(extension);
	
		if(where == start || *(where - 1) == ' ')
		{
			if(*terminator == ' ' || *terminator == '\0') return true;
			start = terminator;
		}
	}

	char buff[100];
	sprintf(buff, "Necessary extensions were not supported: %s", extension);
  printf(buff);
	//MessageBox(NULL, buff, "ERROR", MB_OK | MB_ICONERROR);

	return false;
}


bool initShaderExtension(const char *extension)
{
	if(!generateShaderExtension(extension)) 
	{
		char buff[100];
		sprintf(buff, "Failed to initialize: %s", extension);
    printf(buff);
		return false;
	}
	return true;
}



}
}
#endif // _WIN32
