/**********************************************************************
 * 
 *  File: Texture1.cpp 
 * 
 *  Created: 22. September 2005
 *
 *  Version: $Id: Texture1D.cpp,v 1.2 2005/10/14 13:36:25 trondrh Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	         Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	         Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#include "shallows/Texture1D.hpp"
#include "shallows/shallows.hpp"
#include "shallows/internal/GLIncludes.hpp"
#include "shallows/shallowsexcept.hpp"

namespace shallows {

Texture1D::Texture1D(int internalFormat, 
                     unsigned int width, 
                     int format, 
                     int type, 
                     const void *pixels) : Texture()
{
  internalFormat_ = internalFormat;
  width_ = width;

  glBindTexture(GL_TEXTURE_1D, textureID_);

  glTexImage1D(GL_TEXTURE_1D, 0, internalFormat, width,  
    0, format, (GLenum) type, pixels);

  try 
  {
    CHECK_GL;
  } 
  catch ( ... )
  {
    if (textureID_!=0)
      glDeleteTextures(1, &textureID_);
    textureID_=0;
    throw;
  }
}

Texture1D::Texture1D(int internalFormat, 
   	                 unsigned int width, 
		                 unsigned int textureID) : 
                     Texture(textureID), internalFormat_(internalFormat), width_(width)
{
}


Texture1D::~Texture1D()
{
}

unsigned int Texture1D::getWidth() const
{
  return width_;
}


int Texture1D::getInternalFormat() const
{
  return internalFormat_;
}

void Texture1D::bind() const
{
  glBindTexture(GL_TEXTURE_1D, textureID_);
}
  
void Texture1D::validateSamplerType(const std::string &s) const
{
  if (s!="sampler1D")
  {
    std::string str = "Texture1D attempted to bound to sampler type ";
    str += s;
    throw std::invalid_argument( str );
  }
}


} // end namespace Shallows
