/**********************************************************************
 * 
 *  File: Texture2D.cpp 
 * 
 *  Created: 23. June 2005
 *
 *  Version: $Id: Texture2D.cpp,v 1.10 2005/09/26 12:03:59 jonhjelm Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	         Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	         Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#include "shallows/Texture2D.hpp"
#include "shallows/shallows.hpp"
#include "shallows/internal/GLIncludes.hpp"
#include "shallows/shallowsexcept.hpp"

namespace shallows {

Texture2D::Texture2D(int internalFormat, 
                     unsigned int width, 
                     unsigned int height, 
                     int format, 
                     int type, 
                     const void *pixels,
		     bool rectanguar) : Texture(), rectanguar_(rectanguar)
{
  internalFormat_ = internalFormat;
  width_ = width;
  height_ = height;

  if (rectanguar_==false)
  {
     glBindTexture(GL_TEXTURE_2D, textureID_);

     glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, width, height,
		  0, format,	(GLenum) type, pixels);
  } else
  {
     glBindTexture(GL_TEXTURE_RECTANGLE_ARB, textureID_);

     glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, internalFormat, width, height,
		  0, format,	(GLenum) type, pixels);
  }
  try 
  {
    CHECK_GL;
  } 
  catch ( ... )
  {
    if (textureID_!=0)
      glDeleteTextures(1, &textureID_);
    textureID_=0;
    throw;
  }
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP); 
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
   

}

Texture2D::Texture2D(int internalFormat, 
		     unsigned int width, 
		     unsigned int height,
		     unsigned int textureID,
		     bool rectanguar) : 
   Texture(textureID), internalFormat_(internalFormat),
   width_(width), height_(height), rectanguar_(rectanguar)
   

{
}


Texture2D::~Texture2D()
{
}


bool Texture2D::isRectangular() const
{
   return rectanguar_;
}

unsigned int Texture2D::getHeight() const
{
  return height_;
}

unsigned int Texture2D::getWidth() const
{
  return width_;
}


int Texture2D::getInternalFormat() const
{
  return internalFormat_;
}

void Texture2D::bind() const
{
   if (rectanguar_)
      glBindTexture(GL_TEXTURE_RECTANGLE_ARB, textureID_);
   else
      glBindTexture(GL_TEXTURE_2D, textureID_);
}
  
void Texture2D::validateSamplerType(const std::string &s) const
{
   if (rectanguar_)
   {
      if (s!= "sampler2DRect" && s!="samplerRECT") 
      {
	 std::string str = "TextureRectangle attempted to bound to sampler type ";
	 str += s;
	 throw std::invalid_argument( str );
      }
   } else 
   {
      if (s!="sampler2D")
      {
	 std::string str = "Texture2D attempted to bound to sampler type ";
	 str += s;
	 throw std::invalid_argument( str );
      }
   }
}


} // end namespace Shallows
