/**********************************************************************
 * 
 *  File: Texture3D.cpp
 * 
 *  Created: 22. September 2005
 *
 *  Version: $Id$
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	         Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	         Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#include "shallows/Texture3D.hpp"
#include "shallows/shallows.hpp"
#include "shallows/internal/GLIncludes.hpp"

namespace shallows {
using namespace shallows::internal;
	
Texture3D::Texture3D(int internalFormat, 
	unsigned int width, 
	unsigned int height, 
	unsigned int depth, 
	int format, 
	int type, 
	const void *pixels ) : Texture () {
		
	internalFormat_ = internalFormat;
	width_ = width;
	height_ = height;
	depth_ = depth;
	
	glBindTexture( GL_TEXTURE_3D, textureID_ );
	glTexImage3D( GL_TEXTURE_3D, 0, internalFormat_, width, height, depth, 
		0,format, static_cast<GLenum>( type ), pixels );
	
	try 
  {
    CHECK_GL;
  } 
  catch ( ... )
  {
    if (textureID_!=0)
      glDeleteTextures(1, &textureID_);
    textureID_=0;
    throw;
  }
  glTexParameteri( GL_TEXTURE_3D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri( GL_TEXTURE_3D, GL_TEXTURE_WRAP_T, GL_CLAMP); 
  glTexParameteri( GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri( GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  
}

Texture3D::Texture3D(int internalFormat, 
	unsigned int width, 
	unsigned int height, 
	unsigned int depth, 
	unsigned int textureID) :
	Texture( textureID ), 
	internalFormat_( internalFormat ),
	width_( width ),
	height_( height ),
	depth_( depth )
{}

Texture3D::~Texture3D()
{
}

unsigned int Texture3D::getHeight() const
{
  return height_;
}

unsigned int Texture3D::getWidth() const
{
  return width_;
}


unsigned int Texture3D::getDepth() const
{
  return depth_;
}


int Texture3D::getInternalFormat() const
{
  return internalFormat_;
}

void Texture3D::bind() const
{
  glBindTexture(GL_TEXTURE_3D, textureID_);
}

void Texture3D::validateSamplerType(const std::string &s) const
{
  if (s!="sampler3D")
  {
    std::string str = "Texture3D attempted to bound to sampler type ";
    str += s;
    throw std::invalid_argument( str );
  }
}


} // end namespace Shallows
