/**********************************************************************
 * 
 *  File: imageCreation.cpp
 * 
 *  Created:
 *
 *  Version: $Id: imageCreation.cpp,v 1.3 2005/09/06 14:35:23 josse77 Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#include "shallows/utils/imageCreation.hpp"
#include "shallows/internal/imageCreationInternal.hpp"
#include "shallows/utils/utilsexcept.hpp"

namespace shallows {
namespace utils {

boost::shared_ptr<Image<float> >
getFloatImage( std::string filename, unsigned int channels) {
#ifdef HAVE_MAGICK
    return shallows::internal::genIMImage<float>( filename, channels );
#elif HAVE_PNG
    return shallows::internal::genPngImage<float> ( filename, channels );
#else
    throw not_supported( "Shallows is compiled without support for loading images" );
#endif
}
       

boost::shared_ptr<Image<int> >
getIntImage( std::string filename, unsigned int channels) {
#ifdef HAVE_MAGICK
    return shallows::internal::genIMImage<int>( filename, channels );
#elif HAVE_PNG
    return shallows::internal::genPngImage<int> ( filename, channels );
#else
    throw not_supported( "Shallows is compiled without support for loading images" );
#endif
}

boost::shared_ptr<Image<char> >
getCharImage( std::string filename, unsigned int channels) {
#ifdef HAVE_MAGICK
    return shallows::internal::genIMImage<char>( filename, channels );
#elif HAVE_PNG
    return shallows::internal::genPngImage<char> ( filename, channels );
#else
    throw not_supported( "Shallows is compiled without support for loading images" );
#endif
}

void
writeImageToFile( std::string filename, const utils::Image<float>& image ) {
#ifdef HAVE_MAGICK
    shallows::internal::writeImageToFile( image, filename );
#else
    throw not_supported( "Shallows is compiled without support for loading images" );
#endif
}

}; // namespace utils
}; // namespace shallows
