/**********************************************************************
 * 
 *  File: file
 * 
 *  Created: Jul 14, 2005
 *
 *  Version: $Id: imageCreationInternal.cpp,v 1.2 2005/07/18 13:08:32 johans Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/
#include "shallows/internal/imageCreationInternal.hpp"
#include "shallows/internal/GLIncludes.hpp"
#include "shallows/utils/utilsexcept.hpp"

namespace shallows {
namespace internal {

#ifdef HAVE_MAGICK

template<>
Magick::StorageType 
type2IMenum<float>::operator()() {
    return Magick::FloatPixel;
}


template<>
Magick::StorageType 
type2IMenum<int>::operator()() {
    return Magick::IntegerPixel;
}

template<>
Magick::StorageType 
type2IMenum<short>::operator()() {
    return Magick::ShortPixel;
}

template<>
Magick::StorageType 
type2IMenum<char>::operator()() {
    return Magick::CharPixel;
}

std::string
channelsToIMString( const unsigned int channels ) {
    switch (channels) {
        case 1:
            return "R";
        case 2:
            return "RG";
        case 3:
            return "RGB";
        case 4:
            return "RGBA";
        default:
            throw shallows::utils::not_supported( "ImageMagick only support up to 4 channels. " );
    }
} 

#endif // HAVE_MAGICK

#ifdef HAVE_PNG

template<>
int
type2PngTransforms<char>( unsigned int channels ) {
    int trans = PNG_TRANSFORM_PACKING | PNG_TRANSFORM_STRIP_16;
    
    switch (channels) {
    case 3:
        trans |= PNG_TRANSFORM_STRIP_ALPHA;
        break;
    default:
        break;
    }
    
    return trans;   
}

template<>
int
type2PngTransforms<short>( unsigned int channels ) {
    int trans = PNG_TRANSFORM_PACKING;

    switch (channels) {
    case 3:
        trans |= PNG_TRANSFORM_STRIP_ALPHA;
        break;
    default:
        break;
    }
    
    return trans;   
}

template<>
int
type2PngTransforms<int>( unsigned int channels ) {
    int trans = PNG_TRANSFORM_PACKING;

    switch (channels) {
    case 3:
        trans |= PNG_TRANSFORM_STRIP_ALPHA;
        break;
    default:
        break;
    }
    
    return trans;   
}

#endif // HAVE_PNG
} // namespace internal
} // namespace shallows
