/**********************************************************************
 * 
 *  File: readBuffer.hpp
 * 
 *  Created:
 *
 *  Version: $Id: readBuffer.cpp,v 1.13 2006/01/26 15:16:51 dyken Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#include <vector>

#include "shallows/utils/readBuffer.hpp"
#include "shallows/shallows.hpp"
#include "shallows/FrameBuffer.hpp"
#include "shallows/RenderTarget.hpp"
#include "shallows/internal/GLIncludes.hpp"
#include "boost/shared_ptr.hpp"
namespace shallows {
namespace utils {

void readFloatBuffer(shallows::FrameBuffer &fb, 
		     boost::shared_ptr<RenderTarget> rt, 
		     int components, unsigned int width, unsigned int height,
		     std::vector<float> &pixels)
{
   GLenum format;
   if (components == 1)
     format=GL_ALPHA;
   else if (components == 2)
      //      format=GL_LUMINANCE_ALPHA;
      format=GL_RGB;
   else if (components == 3)
     format=GL_RGB;
   else if (components == 4)
     format=GL_RGBA;
   else
     throw std::invalid_argument
       ( "readFloatBuffer called with invalid number of components" );
   pixels.resize(height*width*components);
   fb.validateRenderTarget(*rt);

   OffScreenBuffer *off_buf = 
      dynamic_cast<OffScreenBuffer*>(&fb);
   if (off_buf!=0)
   {
      boost::shared_ptr<OffScreenRenderTarget> off_rt;
      off_rt=boost::dynamic_pointer_cast<OffScreenRenderTarget> (rt);
      off_buf->attachRenderTarget(off_rt);
   } else
   {
      const OnScreenRenderTarget *on_rt = 
	 dynamic_cast<OnScreenRenderTarget *>(rt.get());
      glReadBuffer(on_rt->getLastBufferEnum());
   }
   if (components==2)
   {
      std::vector<float> tmp(height*width*3);
      glReadPixels(0,0, width, height, format,
		   GL_FLOAT, &tmp[0]);
      for (int i=0; i<(int)(height*width); i++)
      {
        pixels[2*i]=tmp[3*i];
        pixels[2*i+1]=tmp[3*i+1];
      }
   } else
      glReadPixels(0,0, width, height, format,
		   GL_FLOAT, &pixels[0]);
   CHECK_GL;
}

void 
readUnsignedCharBuffer(FrameBuffer &fb, boost::shared_ptr<RenderTarget> rt,
		       int components, unsigned int width, unsigned int height,
		       std::vector<unsigned char> &pixels)
{
   GLenum format;
   if (components == 1)
     format=GL_ALPHA;
   else if (components == 2)
      //      format=GL_LUMINANCE_ALPHA;
      format=GL_RGB;
   else if (components == 3)
     format=GL_RGB;
   else if (components == 4)
     format=GL_RGBA;
   else
     throw std::invalid_argument
       ( "readUnsignedCharBuffer called with invalid number of components" );
   pixels.resize(height*width*components);
   fb.validateRenderTarget(*rt);

   OffScreenBuffer *off_buf = 
      dynamic_cast<OffScreenBuffer *>(&fb);
   if (off_buf!=0)
   {
      boost::shared_ptr<OffScreenRenderTarget> off_rt;
      off_rt=boost::dynamic_pointer_cast<OffScreenRenderTarget> (rt);
      off_buf->attachRenderTarget(off_rt);
   } else
   {
      const OnScreenRenderTarget *on_rt = 
	 dynamic_cast<OnScreenRenderTarget *>(rt.get());
      glReadBuffer(on_rt->getLastBufferEnum());
   }
   if (components==2)
   {
      std::vector<unsigned char> tmp(height*width*3);
      glReadPixels(0,0, width, height, format,
		   GL_UNSIGNED_BYTE, &tmp[0]);
      for (int i=0; i<(int)(height*width); i++)
      {
	 pixels[2*i]=tmp[3*i];
	 pixels[2*i+1]=tmp[3*i+1];
      }
   } else
   glReadPixels(0,0, width, height, format,
		  GL_UNSIGNED_BYTE, &pixels[0]);
   CHECK_GL;
}



} // namespace utils 
} // namespace shallows 
