/**********************************************************************
 * 
 *  File: shallows.cpp
 * 
 *  Created:
 *
 *  Version: $Id: shallows.cpp,v 1.14 2005/09/14 16:08:31 trondrh Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/
#include "shallows/shallows.hpp"
#include "shallows/internal/GLIncludes.hpp"
#include "shallows/internal/ShallowsExt.hpp"
#include "shallows/shallowsexcept.hpp"

#include <iostream>
#include <GL/glu.h>

namespace shallows {

void
setFixedFunction() {
    using namespace internal;
    glUseProgramObjectARB(0);
}

void
init_shallows() {
   if (internal::initShaderExtensions()==false)
      throw missing_extensions("OpenGL extension missing");
}

void 
check_GL( std::string s, int line ) throw ( std::runtime_error ) {
	using std::cerr;
	using std::endl;
	const GLenum err = glGetError();
	if ( err != GL_NO_ERROR ) {
//    	   if ( line != 0 )
//    	      cerr << s << ": at line: " << line << " ";
//    	   cerr << "OpenGL error: " << gluErrorString( err ) << endl;
	   std::string s="OpenGL Error: ";
	   const char * err_c=(const char *)gluErrorString( err );
	   if (err_c)
	     s += err_c;
	   throw std::runtime_error( s );
	}
}	

void 
check_Cg( std::string s, int line ) throw ( std::runtime_error ) {
#if defined(HAVE_CG) && defined(HAVE_CGGL)
    using std::cerr;
    using std::endl;
    const CGerror err = cgGetError();
    if ( err != CG_NO_ERROR ) {
//         if ( line != 0 )
//  	  cerr << s << ": at line: " << line << " ";
//         cerr << "Cg-runtime error: " << cgGetErrorString( err ) << endl;
       std::string s="Cg-runtime Error!";
       const char * err_c=(const char *)cgGetErrorString( err );
       if (err_c)
	  s += err_c;
       throw std::runtime_error( s );
    }
#endif
}

} // namespace shallows
