/**********************************************************************
 * 
 *  File: ImageTest.cpp
 * 
 *  Created: Jul 13, 2005
 *
 *  Version: $Id: ImageTest.cpp,v 1.5 2005/09/20 11:59:35 josse77 Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/
#include "ImageTest.hpp"
#include "shallows/utils/Image.hpp"
#include "shallows/Texture2D.hpp"
#include "shallowstesting.hpp"
#include "shallows/utils/imageCreation.hpp"
#include "shallows/internal/imageCreationInternal.hpp"
#include "shallows/internal/GLIncludes.hpp"

#include <vector>
#include <iterator>
#include <iostream>

using namespace std;

namespace test
{

using namespace shallows;
using shallows::utils::Image;

CPPUNIT_TEST_SUITE_REGISTRATION( ImageTest );

ImageTest::ImageTest()
{
}

ImageTest::~ImageTest()
{
}

void
ImageTest::setUp() {
    initGlut();
}

void
ImageTest::construction() {
    
    vector<float> data( 64*64, 1.0 );
    CPPUNIT_ASSERT_NO_THROW( shallows::utils::Image<float> I = shallows::utils::Image<float>( 64, 64, 1, data ) );
    
}

void
ImageTest::toTexture() {
    vector<float> data( 64*64*4, 1.0 );
    shallows::utils::Image<float> I = shallows::utils::Image<float>( 64, 64, 4, data );
    
    CPPUNIT_ASSERT_NO_THROW( boost::shared_ptr<shallows::Texture2D> t = 
        I.createTexture( GL_RGBA_FLOAT32_ATI) );
    
}

void
ImageTest::loadImage() {
#ifdef HAVE_MAGICK
    CPPUNIT_ASSERT_NO_THROW( {
    boost::shared_ptr<Image<float> > I = 
        shallows::utils::getFloatImage( "images/www.png", 4 );
    boost::shared_ptr<shallows::Texture2D> t = 
        I->createTexture( GL_RGBA_FLOAT32_ATI );        
    } )
    
    CPPUNIT_ASSERT_NO_THROW( {
    boost::shared_ptr<Image<int> > I = 
        shallows::utils::getIntImage( "images/www.png", 4 );
    boost::shared_ptr<shallows::Texture2D> t = 
        I->createTexture( GL_RGBA_FLOAT32_ATI );        
    } )
    
    CPPUNIT_ASSERT_NO_THROW( {
    boost::shared_ptr<Image<char> > I = 
        shallows::utils::getCharImage( "images/www.png", 4 );
    boost::shared_ptr<shallows::Texture2D> t = 
        I->createTexture( GL_RGBA_FLOAT32_ATI );        
    } )
#endif
}

void
ImageTest::pngLibTest() {
#ifdef HAVE_PNG
    CPPUNIT_ASSERT_THROW( shallows::internal::genPngImage<short>(
        "bogusfilename.crazyImageFormat", 4 ), shallows::file_not_found );
    
    CPPUNIT_ASSERT_THROW( shallows::internal::genPngImage<short>( "images/bubble-1.gif", 4 ),
        shallows::utils::not_supported );
    
    CPPUNIT_ASSERT_NO_THROW( shallows::internal::genPngImage<short>( "images/www.png", 4 ) );
    CPPUNIT_ASSERT_NO_THROW( shallows::internal::genPngImage<char>( "images/www.png", 4 ) );
    CPPUNIT_ASSERT_NO_THROW( shallows::internal::genPngImage<int>( "images/www.png", 4 ) );
    // lena-color-512x512-16bit.png is 16bit png (obviously :-) )
    
    const string lena = "images/lena-color-512x512-16bit.png";
    CPPUNIT_ASSERT_NO_THROW( shallows::internal::genPngImage<short>( lena, 4 ) );
    CPPUNIT_ASSERT_NO_THROW( shallows::internal::genPngImage<char>( lena, 4 ) );
#else 
  CPPUNIT_FAIL( "shallows is not compiled with pnglib support, so test can not be run. " );
#endif // HAVE_PNG
}

void 
ImageTest::pngShortToInt() {
#ifdef HAVE_PNG
    using namespace std;
    using namespace shallows::internal;
    using shallows::utils::Image;
    using boost::shared_ptr;

    typedef short Small;
    typedef int Large;
    // This is 16-bit version of lena
    const string lena = "images/lena-color-512x512-16bit.png";
        
    shared_ptr<Image<Small> > smallimg  =  genPngImage<Small>( lena, 4 );
    shared_ptr<Image<Large> > largeimg  =  genPngImage<Large>( lena, 4 );

  /*  CPPUNIT_ASSERT( equal( smallimg->data_.begin(), smallimg->data_.end(), 
    		   static_cast<vector<Small>::const_iterator>( largeimg->data_.begin() ) ) ); 
*/
#else 
  CPPUNIT_FAIL( "shallows is not compiled with pnglib support, so test can not be run. " );
#endif // HAVE_PNG

}

} // namespace test
