/**********************************************************************
 * 
 *  File: file
 * 
 *  Created: Jul 13, 2005
 *
 *  Version: $Id: ImageTest.hpp,v 1.3 2005/07/18 14:37:34 johans Exp $
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/
#ifndef _IMAGETEST_HPP_
#define _IMAGETEST_HPP_

#include <cppunit/extensions/HelperMacros.h>
#include <boost/shared_ptr.hpp>
#include "shallows/utils/Image.hpp"
#include "shallows/internal/imageCreationInternal.hpp"

#include <vector>
#include <iterator>
#include <iostream>
#include <algorithm>

namespace test
{

class ImageTest : public CPPUNIT_NS::TestFixture 
{
    CPPUNIT_TEST_SUITE( ImageTest );
    CPPUNIT_TEST( construction );
    CPPUNIT_TEST( toTexture );
    CPPUNIT_TEST( loadImage );
    CPPUNIT_TEST( pngLibTest );
    //PUNIT_TEST( checkDisplay<char> ); 
    //PUNIT_TEST( checkDisplay<short> );
    //PUNIT_TEST( checkDisplay<int> );
    //CPPUNIT_TEST( pngShortToInt );
    CPPUNIT_TEST_SUITE_END();
public:
	ImageTest();
	virtual ~ImageTest();
    
    void setUp();
    
protected:
    void construction();
    void toTexture();
    void loadImage();
    void pngLibTest();
    
    /** Check that the upcasting works correctly. */
    void pngShortToInt();
    
    template<typename T>
    void checkDisplay();
    
};

template<typename T>
void
ImageTest::checkDisplay() {
#ifdef HAVE_MAGICK
#ifdef HAVE_PNG
    
    using namespace std;
    using namespace shallows::internal;
    using shallows::utils::Image;
    using boost::shared_ptr;
    shared_ptr<Image<T> > img  =  genPngImage<T>( "images/johan.png", 4 );

    Magick::Image I( img->width_, img->height_, channelsToIMString( img->channels_ ),
        type2IMenum<T>()(), &img->data_[0] );
    
    //copy( img->data_.begin(), img->data_.end(), ostream_iterator<T>( cout, " " ) );
    I.display();

#endif // HAVE_PNG
#else 
    CPPUNIT_FAIL( "shallows must be compiled with both pnglib and ImageMagick for this test" );
#endif // HAVE_MAGICK
   
}

};


#endif //_IMAGETEST_HPP_
