/**********************************************************************
 * 
 *  File: 
 * 
 *  Created:
 *
 *  Version: $Id$
 *
 *  Authors: Trond R. Hagen <trr@sintef.no>, 
 *	     Jon Mikkelsen Hjelmervik <jami@sintef.no>,
 *	     Johan S. Seland <johans@ifi.uio.no>
 *
 *  This file is part of the Shallows library.
 *  Copyright (C) 2005 by SINTEF.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  ("GPL") version 2 as published by the Free Software Foundation.
 *  See the file LICENSE.GPL at the root directory of this source
 *  distribution for additional information about the GNU GPL.
 * 
 *  For using Shallows with software that can not be combined with the
 *  GNU GPL, please contact SINTEF for aquiring a commercial license
 *  and support.
 *
 *  SINTEF, Pb 124 Blindern, N-0314 Oslo, Norway
 *  http://www.sintef.no
 *********************************************************************/

#include "Texture3DTest.hpp"
#include "shallows/shallows.hpp"
#include "shallows/utils/utils.hpp"
#include "shallows/internal/GLIncludes.hpp"

#include "shallowstesting.hpp"
#include <string>
#include <cassert> 
#include <iostream>
#include <stdexcept>
#include <vector>
#include <iterator>
using namespace std;
using boost::shared_ptr;

namespace test
{

CPPUNIT_TEST_SUITE_REGISTRATION( Texture3DTest );

Texture3DTest::Texture3DTest()
{
}

Texture3DTest::~Texture3DTest()
{
}


void
Texture3DTest::setUp() {
	initGlut();
}

void
Texture3DTest::construction() {
   CPPUNIT_ASSERT_NO_THROW( shallows::Texture3D tex(GL_RGBA_FLOAT32_ATI,
						    4, 4, 4, 
						    GL_RGBA,
						    GL_FLOAT, 
						    (const float *)0));
   CPPUNIT_ASSERT_THROW( shallows::Texture3D tex(GL_FLOAT,
						 4, 4, 4,
						 GL_RGBA,
						 GL_FLOAT, 
						 (const float *)0),
			 std::runtime_error);
}

void Texture3DTest::render() {
	
	shallows::GLProgram shader;
	shader.readFile( "shaders/Sum3DTexture.glshader" );
	
	vector<float> data;
	for ( size_t i = 0; i < 4; ++i )
		for ( size_t j = 0; j < 4; ++j )
			for ( size_t k = 0; k < 4; ++k ) {
				data.push_back( i );
				data.push_back( i );
				data.push_back( i );
				data.push_back( i );
			}
			
	shared_ptr<shallows::Texture3D> tex( new shallows::Texture3D( GL_RGBA_FLOAT32_ATI, 
		4, 4, 4, GL_RGBA, GL_FLOAT, &data[0] ) );
	
	shader.setInputTexture( "texture", tex );
	
	
   	boost::shared_ptr<shallows::OffScreenBuffer> fb ( new shallows::OffScreenBuffer( 4, 4 ));
   	boost::shared_ptr<shallows::RenderTexture2D> rt( fb->createRenderTexture2D() );

    shader.setFrameBuffer(fb);
    shader.setOutputTarget(0, rt);

    shader.run();
    
    vector<float> float_pixels;
    
    shallows::utils::readFloatBuffer(*fb, rt, 4, 4, 4, float_pixels);
    
//    copy( float_pixels.begin(), float_pixels.end(), 
//    	ostream_iterator<float>( cout, " " ) );
	for( vector<float>::const_iterator i = float_pixels.begin();
		i != float_pixels.end(); ++i )
		CPPUNIT_ASSERT_EQUAL( static_cast<double>( *i ), 6.0 );
		
    
}

} // namespace test


