//Includes the needed file from standard libary
#include <cstdio>
#include <vector>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <sstream>
#include <GL\glew.h>
#include <GL\glut.h>
#include <cmath>
#include <algorithm>
#include <glext.h>

#ifdef _WIN32
#include <windows.h>
#endif

#define BUFFER_OFFSET(i) ((char *)NULL + (i));

//Includes the needed file from shallows and boost libary
#include "shallows/GLProgram.hpp"
#include "shallows/OnScreenBuffer.hpp"
#include "shallows/OffScreenBuffer.hpp"
#include "shallows/OnScreenRenderTarget.hpp"
#include "shallows/OffScreenRenderTarget.hpp"
#include "shallows/RenderTexture2D.hpp"
#include "shallows/utils/textureCreation.hpp"
#include "shallows/utils/utils.hpp"
#include "shallows/shallowsexcept.hpp"
#include "shallows/shallows.hpp"
#include "boost/shared_ptr.hpp"

//Specify which function to use from the libarys
using shallows::Texture1D;
using shallows::Texture2D;
using shallows::GLProgram;
using shallows::OnScreenBuffer;
using shallows::OnScreenRenderTarget;
using shallows::OffScreenBuffer;
using shallows::OffScreenRenderTarget;
using shallows::RenderTexture2D;
using shallows::utils::createFloatTexture1D;
using shallows::utils::createFloatTexture2D;
using boost::shared_ptr;

using namespace shallows;
using namespace std;

//Define the computional grid
const unsigned int BUFF_DIM = 48;

//Arrays for holding values from textfile, arrays for init values and for bottoms
//float SSPCurr[4096];
//float SSPNext[4096];
float SSPCurr[2048];
float SSPNext[2048];
float SSGCurr[2048];
float SSGNext[2048];
float setvalues[BUFF_DIM*BUFF_DIM*4];
float init[BUFF_DIM*BUFF_DIM*4];
float init2[BUFF_DIM*BUFF_DIM*4];
float bottom[2048];
float bottom2[2048];
float Pi = 3.14159265;
int Depth;

//Defines the window width and height
int window_width;
int window_height;

//Makes the pointer to the shader used in the program
shared_ptr<GLProgram> RayShader;
shared_ptr<GLProgram> RenderShader;
shared_ptr<GLProgram> InitProg;

//Pointer for frambuffer readback
shared_ptr<OffScreenBuffer> offs_fb;
shared_ptr<OnScreenBuffer> ons_fb;
shared_ptr<OnScreenRenderTarget> ons_rt;

//Makes four rendertarget, and gives them names
shared_ptr<RenderTexture2D> rt[4];
shared_ptr<RenderTexture2D> curr1, next1, curr2, next2;

//Timing the application
shallows::utils::Timer rolex;

//Vectors for feedback form shader and for visualizing
std::vector<float> readback1;
std::vector<float> readback2;
std::vector<float> readback3;
std::vector<float> readback4;
std::vector<float> readback5;
std::vector<float> readback6;
std::vector<float> vertz;
std::vector<float> texCoords;
std::vector<unsigned int> indices;
std::vector<float> store;
std::vector<std::vector<float> > save;

bool pause;
bool clear;
bool read;


/*This is a basic reshape function. Here the projection matrix is
  not set, only relaying on the identity matrix*/
void reshape(int width, int height)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glViewport(0, 0, width, height);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

/*This calls the above reshape function. One might want to reshape both the 
  OnScreenBuffer and OffScreenBuffer. Since they do not share the same size 
  it is nice to split them up into two different functions.*/
void callback_reshape(int width, int height)
{
	window_width = width;
	window_height = height;
	reshape(window_width, window_height);
}

/*This is the keyboard function. If q are pressed the application will close*/
void keyboard(unsigned char key, int winx, int winy)
{
	switch(key)
	{
		case 'q':
			exit(0);			
	}
	switch(key)
	{
		case 'p':
			pause = true;
	}
	switch(key)
	{
		case 's':
			pause = false;
	}
	switch(key)
	{
		case 'r':
			read = true;
	}
	switch(key)
	{
		case 'a':
			read = false;
	}

}

/* This function just draws a nice string on the current framebuffer. Useful
 * for FPS counters and information. */
void renderBitmapString( float x, float y, float z, const string s) 
{  
  glRasterPos3f(x, y, z);
  for ( string::const_iterator it = s.begin(); it != s.end(); ++it ) 
  {
    glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *it);
  }
}

/*Reads the SoundSpeedProfile.dat and stores it in an array called SSP*/
void fileRead()
{

	int i;

	//Setting all values in the array to 0.0
	for(i=0; i<2048; i++)
	{
		SSGNext[i] = 0.0f;
	}
		for(i=0; i<2048; i++)
	{
		SSGCurr[i] = 0.0f;
	}

	fstream InFile;
	float Total;
	int Count;

	InFile.open("SoundSpeedProfile5.dat", ios::in);
	if (InFile.fail())
	{
		cout << "Could not open the sound speed profile file" << endl;
		exit(1);
    }

	Count = 0;
	Total = 0.0;

	InFile >> SSGCurr[512];

	i = 0;
	while (! InFile.fail())
	{
		InFile >> SSGCurr[i];
		i++;
    }
 
	//cout << SSGCurr[512] << endl;

	ofstream Out("Output.dat");
	for(int z=0; z <= 512; z++)
	{
		Out << SSGCurr[z] << endl;
	}

	InFile.close();
	
	/*Interpolation routine to get enough values to decrease layer.*/
	float a;
	float deltalayer[512];
	float deltalayerstore[512];
	int delta = 1; //Layer thickness = 1/delta.
	float SSGTotal[1024];
	//for(int k=0; k<512; k++)
	//{
	//	a = max(SSGCurr[k],SSGCurr[k+1])- min(SSGCurr[k],SSGCurr[k+1]);
	//	deltalayer[k] = a/delta;
	//	deltalayerstore[k] = min(SSGCurr[k],SSGCurr[k+1]) + deltalayer[k];
	//	//cout << a << endl;
	//	SSGTotal[k] = SSGCurr[k];
	//	SSGTotal[512+k] = deltalayerstore[k];	
	//}
	//sort(SSGTotal, SSGTotal+1024);

	/*Sound Speed Profile for delta layers.*/
	//for(int j=0; j<=1024; j++)
	//{
	//	SSPCurr[j*4] = SSGTotal[j];
	//}
	//for(int l=0; l<=1024; l++)
	//{
	//	SSPNext[l*4] = SSGTotal[l + 1];
	//	
	//}
	/*Sound Speed Profile from text file.*/
		for(int j=0; j<=512; j++)
	{
		SSPCurr[j*4] = SSGCurr[j];
	}
	for(int l=0; l<=512; l++)
	{
		SSPNext[l*4] = SSGCurr[l + 1];
		
	}
}
/*Creats the initial data for ray calculation. The depth for the source is stored
  in the first value and range for the source is stored in the secound value. This values are
  the same for every calculated ray, which make them start in the same place. The delta for 
  each layer is stored in the third value. The value issame for every layer. The last and fourth value are
  the start for the soundspeedprofile, and the value for this are 1495. All values are stored in array setvalues.
  The next array, init, are the values for angle in radians. This start value are different for every calculation
  point, that is every pixel/ray. */
void initData()
{
	float deltaStep = 1.0f;
	float Degrees = 5.0f;
	float delta = (Degrees*2)/float(BUFF_DIM*BUFF_DIM);
	float temp = 0.0f;
	for(int i=0; i<=BUFF_DIM*BUFF_DIM; i++)
	{
		setvalues[i*4] = 256.0f;				// Depth for source.
		setvalues[i*4+1] = 0.0f;				// Range for source.
		setvalues[i*4+2] = deltaStep;			// Direction of the ray
		setvalues[i*4+3] = 0.0f;				// Travel time, allways starts at zero
	}

	for(int j=0; j<=BUFF_DIM*BUFF_DIM; j++)
	{
		init[j*4] = (Degrees - temp)*Pi/180;			// From angle to angle (within 45 to -45)in radians
		if((Degrees - temp)<0.0)
			setvalues[j*4+2] = -deltaStep;
		temp += delta;
	}

}

/*The creation of bottom profile and the next store values for keeping track of the bottom */
void initBottom()
{
	float x = 0.0f;
	float y = 400.0f;
	float delta2 = 0.0f;
	float delta3 = 0.0f;
	float rangeStep = 0.0f;
	float angle = 0.0f;
	float norm = 1.0f;
	for(int z=0; z<512; z++)
	{
		bottom[z*4] = (x + rangeStep)/norm;
		bottom[z*4+1] = (y + delta2)/norm;

		if (z <= 102)
		{
			delta2 += 0.0;
			angle = atanf((102.0*0.0)/(39.0*102.0));		
		}
		else if ( z <= 204 )
		{
			delta2 += -0.3;
			angle = atanf((102.0*0.3)/(39.0*102.0));		
		}
		else if ( z <= 306)
		{
			delta2 += -0.1;
			angle = atanf((102.0*0.1)/(39.0*102.0));		
		}
		else if (z <= 408)
		{
			delta2 += 0.1;
			angle = atanf((102.0*-0.1)/(39.0*102.0));		
		}
		else
		{
			delta2 += 0.2;
			angle = atanf((104.0*-0.2)/(39.0*104.0));		
		}
		bottom[z*4+2] = angle;
		bottom[z*4+3] = 50.0f;
		delta3 += 0.001;
		rangeStep += 39.0;
	}

	for(int i=0; i<512; i++)
	{
		bottom2[i*4] = bottom[i*4]/20000;
		bottom2[i*4+1] = -bottom[i*4+1]/600;
		bottom2[i*4+2] = bottom[i*4+2];
		bottom2[i*4+3] = bottom[i*4+3]/100;

	}

	for(int k=0; k<=BUFF_DIM*BUFF_DIM; k++)
	{
		init2[k*4] = (cos(init[k*4]))/(SSPCurr[int(setvalues[0])*4]);					
		init2[k*4+1] = 0.0f;				
		init2[k*4+2] = 0.0f;				
		init2[k*4+3] = 0.0f;				
	}
}
void bottomDraw()
{  
	//reshape(window_width, window_height);
	//glPointSize(2.0);
	glLineWidth(2.0);
//	glBegin(GL_LINE_STRIP);
	glColor3f(0.8, 0.4, 0.1);
	glBegin(GL_POLYGON);
		glVertex2f(-0.95, -0.768);
		glVertex2f(-0.95, -0.38);
		for (int i=0; i<512; i++)
		{
			glVertex2f(bottom2[i*4]*2.0 - 0.944, bottom2[i*4+1]*2.0 + 0.945);
		}

		glVertex2f(1.0, -0.768);
		glVertex2f(1.0, -0.4);
	glEnd();
	
	glColor3f(1.0, 0.0, 0.0);
	glBegin(GL_LINES);
		glVertex2f(-0.95, 0.95);
		glVertex2f(-0.95, -0.768);
	glEnd();

	glColor3f(1.0, 0.0, 0.0);
	glBegin(GL_LINES);
		glVertex2f(-0.95, -0.768);
		glVertex2f(1.0, -0.768);
	glEnd();

	//Reciver array at 50 meters
	//glLineWidth(3.0);
	//glColor3f(0.0, 0.0, 0.0);
	//glBegin(GL_LINE_STRIP);
		//glVertex2f(-1.0, 0.5);
		for (int i=0; i<512; i++)
		{
			glVertex2f((i)/1024, bottom2[i*4+3]);
		}

	//glEnd();
}

/*Render timing for application*/
void timing()
{
	/* Here follows just functionality to draw an FPS counter */
    static unsigned int frames = 0;
	static unsigned int count = 0;
    static float fps;
	static float sec;
    static ostringstream os;
	static ostringstream sek;

    os.flags( ios_base::right | ios_base::fixed  );
    os.precision( 1 );
	sek.flags( ios_base::right | ios_base::fixed );
	sek.precision( 1 );
    static string fpsstring1;
	static string fpsstring2;
	static string fpsstring3 = "500";
	//static string fpsstring4 = "0";
	static string fpsstring5 = "40";
	static string fpsstring6 = "Depth[m]";
	static string fpsstring7 = "Range[km]";

    
    frames++;
	count++;
	
    if ( rolex.elapsed() > 0.1 ) {
		fps = static_cast<double>( frames ) / rolex.elapsed();
		frames = 0;
		sec += rolex.elapsed();
   		rolex.restart();
   			
   		os << "FPS: " << fps;
		sek << "Sek: " << sec;

   		fpsstring1 = os.str();
		fpsstring2 = sek.str();
   		os.str("");
		sek.str("");
    }

    	
    setFixedFunction();
	glColor3f( 0.0f, 0.0f, 0.0f);
	renderBitmapString( -0.95, -0.95, 1.0, fpsstring1 );
	renderBitmapString( -0.95, -0.90, 1.0, fpsstring2 );
	renderBitmapString( -1.0, -0.77, 1.0,  fpsstring3 );
	//renderBitmapString( -0.95, -0.80, 1.0, fpsstring4 );
	renderBitmapString( 0.97, -0.80, 1.0, fpsstring5 );
	renderBitmapString( 0.86, -0.84, 1.0, fpsstring7 );
	renderBitmapString( -0.97, 0.97, 1.0, fpsstring6 );
	//renderBitmapString( -0.97, 0.93, 1.0, fpsstring4);
}

/*The main core of the program lies here*/
void initGL()
{
	//Initialize shallows
	shallows::init_shallows();
	
	GLenum err = glewInit();
	if (GLEW_OK != err)
	{
		 /* Problem: glewInit failed, something is seriously wrong. */
		 fprintf(stderr, "Error: %s\n", glewGetErrorString(err));
	}
	fprintf(stdout, "Status: Using GLEW %s\n", glewGetString(GLEW_VERSION));

	pause = false;

	glClearColor(1.0, 1.0, 1.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);
	//Executes the fileRead and initData functions above
	fileRead();
	initData();
	initBottom();
	//Interpolation();
	
	//for(int i=0; i<512; i++)
	//{
	//	cout << "at " << i << " meter the ssp is " << bottom[i] << endl;
	//}
	//for (int i=0; i<=BUFF_DIM*BUFF_DIM; i++)
	//{
	//	cout <<"depth " <<init[i*4] << " range "<<init[i*4+1] <<" angle "<<init[i*4+2] 
	//	     <<" alpha " << init[i*4+3] << endl;
	//}
	
	//Creates textures from the SSP and init array
	boost::shared_ptr<Texture1D> init_texSSPCurr = shallows::utils::createFloatTexture1D(512, 4, SSPCurr);
	boost::shared_ptr<Texture1D> init_texSSPNext = shallows::utils::createFloatTexture1D(512, 4, SSPNext);
	boost::shared_ptr<Texture1D> init_texBottom = shallows::utils::createFloatTexture1D(512, 4, bottom);
	boost::shared_ptr<Texture2D> init_texInit = shallows::utils::createFloatTexture2D(BUFF_DIM, BUFF_DIM, 4, init);
	boost::shared_ptr<Texture2D> init_texInit2 = shallows::utils::createFloatTexture2D(BUFF_DIM, BUFF_DIM, 4, init2);
	boost::shared_ptr<Texture2D> init_texSetValues = shallows::utils::createFloatTexture2D(BUFF_DIM, BUFF_DIM, 4, setvalues);


	/*************** Off-Screen Rendering ************************************/

	//Creates a buffer for off-screen rendering and generate render to texture into it
	//Textures are also renamed
	offs_fb.reset(new OffScreenBuffer(BUFF_DIM, BUFF_DIM));
	rt[0] = offs_fb->createRenderTexture2D();
	rt[1] = offs_fb->createRenderTexture2D();
	rt[2] = offs_fb->createRenderTexture2D();
	rt[3] = offs_fb->createRenderTexture2D();
	curr1 = rt[0];
	next1 = rt[1];
	curr2 = rt[2];
	next2 = rt[3];

	//Creating the shader for off screen rendering and attach it to a frambuffer
	RayShader.reset(new GLProgram);
	RayShader->readFile("rayEquations.glshader");
	RayShader->setFrameBuffer(offs_fb);

	/************** On-Screen Rendering **************************************/

	//Creates a buffer and a render target for on-screen rendering
//	shared_ptr<OnScreenBuffer> ons_fb(new OnScreenBuffer);
//	shared_ptr<OnScreenRenderTarget> ons_rt(new OnScreenRenderTarget(GL_BACK_LEFT));
	ons_fb.reset(new OnScreenBuffer);
	ons_rt.reset(new OnScreenRenderTarget(GL_BACK_LEFT));

	//Creating the shader for on screen rendering and attach it to a frambuffer
	RenderShader.reset(new GLProgram);
	RenderShader->useNormalizedTexCoords();
	RenderShader->readFile("rayRenderer.glshader");
	RenderShader->setFrameBuffer(ons_fb);
	RenderShader->setOutputTarget(0, ons_rt);
	RenderShader->setInputTexture("result", rt[0]->getTexture());

	/*************** Initial values ******************************************/
	//Gets the initial values into the shader
	InitProg.reset(new GLProgram);
	InitProg->useNormalizedTexCoords();
	InitProg->readFile("rayEquations.glshader");
	InitProg->setFrameBuffer(offs_fb);
	InitProg->setInputTexture("texSSPCurr", init_texSSPCurr);		//contains the current step soundspeedprofile
	InitProg->setInputTexture("texSSPNext", init_texSSPNext);		//contains the next step soundspeedprofile
	InitProg->setInputTexture("texBottom", init_texBottom);			//contains the bottom properties
	InitProg->setInputTexture("texInit", init_texInit);				//contains the angles
	InitProg->setInputTexture("texInit2", init_texInit2);			//will keep track of bottom position
	InitProg->setInputTexture("texSetValues", init_texSetValues);	//contains the depth, range, ray direction and travel time 
	//InitProg->setParam1f("sspDepth", SSPCurr[int(setvalues[0])*4]); //sets the soundspeedprofile value for ray start

	reshape(BUFF_DIM, BUFF_DIM);

	//Sets the rendertargets
	InitProg->setOutputTarget(0, curr1);
	InitProg->run();
	shallows::utils::readFloatBuffer(*offs_fb,curr1,4,BUFF_DIM, BUFF_DIM,readback1) ;
	InitProg->setOutputTarget(0, next1);
	InitProg->run();
	shallows::utils::readFloatBuffer(*offs_fb,next1,4,BUFF_DIM, BUFF_DIM,readback2) ;
	InitProg->setOutputTarget(1, curr2);
	InitProg->run();
	//shallows::utils::readFloatBuffer(*offs_fb,curr2,4,BUFF_DIM, BUFF_DIM,readback1) ;
	InitProg->setOutputTarget(1, next2);
	InitProg->run();
	//shallows::utils::readFloatBuffer(*offs_fb,next2,4,BUFF_DIM, BUFF_DIM,readback2) ;
/*
   indices.push_back(0);
   indices.push_back(BUFF_DIM*BUFF_DIM/2);
   indices.push_back(BUFF_DIM*BUFF_DIM/4);
   indices.push_back(BUFF_DIM*BUFF_DIM/6);
   indices.push_back(BUFF_DIM*BUFF_DIM-1);
   indices.push_back(BUFF_DIM*BUFF_DIM-2);
   indices.push_back(BUFF_DIM*BUFF_DIM-5);
*/
	unsigned int counter = 0;
   for(int i=0; i<BUFF_DIM; i++)
   {
	   for(int j=0; j<BUFF_DIM; j++)
	   {
		   //if (j%10==0) 
		   indices.push_back(counter);
		   texCoords.push_back((float)(j)/(float)(BUFF_DIM));
		   texCoords.push_back((float)(i)/(float)(BUFF_DIM));
		   texCoords.push_back(0.0);
		   texCoords.push_back(0.0);
		   vertz.push_back(0.0);
		   vertz.push_back(0.0);
		   vertz.push_back(0.0);
		   counter++;
	   }
   }

   save.resize(BUFF_DIM*BUFF_DIM);

   rolex.restart();
}
/*This is the function where the actual rendering take place*/
void display()
{
	glPointSize(1.0);
	//glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if(!pause)
	{
		
	////unsigned int storeBuffers;
	//GLuint storeBuffers = 0;
	//glGenBuffers(1, &storeBuffers);  

	//glBindBuffer(GL_PIXEL_PACK_BUFFER_ARB, storeBuffers);  
	//glBufferData(GL_PIXEL_PACK_BUFFER_ARB, storeBuffers, NULL, GL_STATIC_READ);  
	
	//// render to framebuffer  
	//glDrawBuffer(GL_BACK);  

	/******* Off-Screen Rendering *****************************/
	//Reshape to fit the computional grid
	reshape(BUFF_DIM, BUFF_DIM);
	
	//Activates the shader
	InitProg->setOutputTarget(0, next1);
	InitProg->setInputTexture("texSetValues", curr1->getTexture());
	InitProg->run();
	shallows::utils::readFloatBuffer(*offs_fb,curr1,4,BUFF_DIM, BUFF_DIM,readback3);
	shallows::utils::readFloatBuffer(*offs_fb,next1,4,BUFF_DIM, BUFF_DIM,readback4);
	InitProg->setOutputTarget(1, next2);
	InitProg->setInputTexture("texInit2", curr2->getTexture());
	InitProg->run();
	shallows::utils::readFloatBuffer(*offs_fb,curr2,4,BUFF_DIM, BUFF_DIM,readback5);
	shallows::utils::readFloatBuffer(*offs_fb,next2,4,BUFF_DIM, BUFF_DIM,readback6);

	//********** Here starts the asynchronus readback *********//

	//// Bind one buffer objects and start the ReadPixels   
	//// asynchronously. The call will return directly after starting the   
	//// DMA transfer.  
	//glBindBuffer(GL_PIXEL_PACK_BUFFER_ARB, storeBuffers);  
	////glReadPixels( 0, 0, BUFF_DIM, BUFF_DIM, GL_BGRA, GL_UNSIGNED_BYTE, BUFFER_OFFSET(0) ); 

	////process partial images   
	//glMapBuffer(GL_PIXEL_PACK_BUFFER_ARB, GL_READ_ONLY);  
	////processImage(pboMemory1);  

	//// unmap the image buffers  
	//glBindBuffer(GL_PIXEL_PACK_BUFFER_ARB, storeBuffers);  
	//if (!glUnmapBuffer(GL_PIXEL_PACK_BUFFER_ARB)) 
	//{  
	//// Handle error case  
	//}  
   	
	store.clear();
	int reciverDepth = 50;

	for (int i=0; i < BUFF_DIM*BUFF_DIM; i++)
	{
		if( readback3[i*4+1] < 10000)
		{
			if ( (readback3[i*4+2] + readback4[i*4+2]) == 0 )
			{
				//Depth for hit (bottom_label / surface_label / turning_below_label / turning_above_label)
				save[i].push_back(readback3[i*4]);	
				//Range to hit from source (bottom_range / surface_range)
				save[i].push_back(readback3[i*4+1]);
				//Angle for bottom when hit (bottom_angle)
				save[i].push_back(init[i*4]-bottom[i*4+2]);	
			}
			if( readback3[i*4] == reciverDepth )
			{
				//Reciver depth
				save[i].push_back(readback3[i*4]);	
				//Range when reciver is hit N time (eig_range)
				save[i].push_back(readback3[i*4+1]);
				//Travel time for ray when N hit (eig_time)
				save[i].push_back(readback3[i*4+3]);
				//Start angle for ray N(eig_theta)
				save[i].push_back(init[i*4]);
				//Angle for ray to reciver when N hit (eig_angle)
				save[i].push_back(readback3[i*4+2]*acos(init2[i*4]*SSPCurr[reciverDepth*4]));
			}
		}
	}

	float k;

	if (read)
	{
	ofstream OutData("ResultValues.dat");

		for(int a = 0; a < save.size(); a++)
		{
			for(int b = 0; b < save[a].size(); b++)
			{
				k = save[a][b];
				
				OutData << "Ray Number: " << a << " Element Number: " << b << " Has value: " << k << endl; 

			}
		}
	OutData.close();

	//ofstream OutBottom("Bottom.dat");
	//
	//	for (int c = 0; c < 2048; c++)
	//	{
	//		OutBottom << bottom[c*4+1] << endl;
	//	}
	//OutBottom.close();

	}

	/******* On-Screen Rendering ******************************/

	//Reshape to fit the screenq
	reshape(window_width, window_height);

	//Activates the shader
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_COLOR_ARRAY);
	RenderShader->setInputTexture("result", curr1->getTexture());
	RenderShader->activate();
	glVertexPointer(3, GL_FLOAT, 0, &vertz[0]);
	glColorPointer(4, GL_FLOAT, 0, &texCoords[0]);
//	glDrawArrays(GL_LINE_STRIP, 0, BUFF_DIM*BUFF_DIM);
	glPointSize(1.0);
	glDrawElements(GL_POINTS, indices.size(), GL_UNSIGNED_INT, &indices[0]);
//	glDrawElements(GL_POINTS, 10, GL_UNSIGNED_INT, &indices[indices.size()/2.0]);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY);
	
	timing();
	bottomDraw();
	
	swap(curr1, next1);
	swap(curr2, next2);

	//Swap the buffer so for the view so it looks like a smooth animation
	glutSwapBuffers();

	}
	//Makes the program use reshape function within next loop
	glutPostRedisplay();

}

/*This is the main function for starting the program. The glut utility libary is 
  used for window creation. The reshape and display function above is called and
  executed here. Also some error handling is used to determine what went wrong,
  when running the core of the program, if something did actually go wrong.
  Information about the graphics card is printed on the screen. In the end the 
  program is ran over and over again. */

int main(int argc, char** argv)
{
	//Initializing GLUT and setting window
	glutInit(&argc, argv);
	glutInitWindowPosition(50, 100);
	glutInitWindowSize(1280, 768);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	int windowID = glutCreateWindow("Plane Ray Using Shallows");
	glutSetWindow(windowID);

	//Calls the above function
	glutReshapeFunc(callback_reshape);
	glutKeyboardFunc(keyboard);
	glutDisplayFunc(display);

	//Error handling
	try{initGL();}
	catch (shallows::compile_error &e)
	{
		std::cout << "Compile error:\n " << e.log() << std::endl;
		exit(EXIT_FAILURE);
	}
	catch (std::exception &e)
	{
		std::cout << e.what() << std::endl;
		exit(EXIT_SUCCESS);
	}

	//Display some additional information on the screen
	cout <<"Graphics chip manufactor: " << glGetString(GL_VENDOR)<< endl
	     <<"Graphics chip: " << glGetString(GL_RENDERER) << endl
	     <<"OpenGL Version: " << glGetString(GL_VERSION) << endl;
		 //<< "Extentions: " << glGetString(GL_EXTENSIONS) << endl;

	//Runs the loop over and over again
	glutMainLoop();

	return EXIT_SUCCESS;
}